# -*- coding: utf_8 -*-
"""
Compare Apps.

Module for comparing app results.
"""
import re
from collections import defaultdict
from copy import deepcopy

from django.shortcuts import render
from django.conf import settings
from django.utils.html import escape

from mobsf.MobSF.utils import print_n_send_error_response
from mobsf.StaticAnalyzer.models import StaticAnalyzerAndroid
from mobsf.StaticAnalyzer.views.android.db_interaction import (
    get_context_from_db_entry,
)


# The APKiD is pain because it really differs from the others
def diff_apkid(context: dict) -> None:
    apkid_keys = ['anti_vm', 'compiler', 'obfuscator',
                  'packer', 'dropper', 'manipulator',
                  'anti_disassembly', 'anti_debug', 'abnormal']

    context['apkid'] = {}
    context['apkid']['common'] = {}
    context['apkid']['only_first'] = {}
    context['apkid']['only_second'] = {}

    first_apkid, second_apkid = (context['first_app']['apkid'],
                                 context['second_app']['apkid'])
    first_error, second_error = first_apkid.get(
        'error', False), second_apkid.get('error', False)
    context['apkid_error'] = (first_error or second_error)
    if context['apkid_error']:
        return

    tmp_flat_results = {}
    for curr_app_str in ['first_app', 'second_app']:
        # Flatten the results per app (join all the dex files result into one)
        flat_results = defaultdict(list)
        for results in [dex_results for (dex_name, dex_results) in
                        context[curr_app_str]['apkid'].items()]:
            for apkid_key, key_results in results.items():
                flat_results[apkid_key] += key_results
                # Distinct list
                flat_results[apkid_key] = list(set(flat_results[apkid_key]))
        tmp_flat_results[curr_app_str] = deepcopy(flat_results)

    for key in apkid_keys:
        # The keys may be different between the two lists
        context['apkid']['common'][key] = [
            x for x in tmp_flat_results['first_app'][key] if x in
            tmp_flat_results['second_app'][key]]

        context['apkid']['only_first'][key] = [
            x for x in tmp_flat_results['first_app'][key] if x not in
            tmp_flat_results['second_app'][key]]

        context['apkid']['only_second'][key] = [
            x for x in tmp_flat_results['second_app'][key] if x not in
            tmp_flat_results['first_app'][key]]


def diff_browsable_activities(context, first_app, second_app):
    # Browsable activities are 'complex' objects that contains additional
    # fields that should be compared. The generic diffing performed above
    # only list which browsable activities are in common or present only in
    # one app. For the activities present in one app we can just pretty-print
    # the details but for common browsable activities we need to perform the
    # diffing also on sub-keys, like hosts and schemes.
    section = 'browsable_activities'
    common_activity = 'common_browsable_activities'
    browsable_activities_keys = ['schemes', 'mime_types', 'hosts', 'ports',
                                 'paths', 'path_prefixs', 'path_patterns']

    for act, _ in context[section]['common']:
        context[common_activity][act] = {}
        for key in browsable_activities_keys:
            context[common_activity][act][key] = {}

            context[common_activity][act][key]['common'] = [
                y for y in first_app[section][act][key] if y in
                second_app[section][act][key]
            ]

            context[common_activity][act][key]['only_first'] = [
                y for y in first_app[section][act][key] if y not in
                second_app[section][act][key]
            ]

            context[common_activity][act][key]['only_second'] = [
                y for y in second_app[section][act][key] if y not in
                first_app[section][act][key]
            ]


# suppose to get any 2 apps (android / ios / appx)
# and then figure out what to do with them
def generic_compare(request,
                    first_hash: str,
                    second_hash: str,
                    api: bool = False):
    # This context consists of specific lists and analysis
    # that is done on the classic ones
    # it will be filled during the different diff analysis
    context = {
        'title': 'Compare report',
        'version': settings.MOBSF_VER,
        'first_app': {},
        'second_app': {},
        'urls': {},
        'android_api': {},
        'permissions': {},
        'browsable_activities': {},
        'common_browsable_activities': {},
        'apkid': {},
    }
    static_fields = ['md5', 'file_name', 'size', 'icon_path',
                     'activities', 'services', 'providers',
                     'receivers', 'exported_count', 'apkid']

    # For now - support only android
    db_entry = StaticAnalyzerAndroid.objects.filter(MD5=first_hash)
    db_entry2 = StaticAnalyzerAndroid.objects.filter(MD5=second_hash)

    if not (db_entry.exists() and db_entry2.exists()):
        return print_n_send_error_response(
            request,
            'Currently you can only diff/compare android apps. '
            'One of the app has not completed static analysis or'
            ' they are not both android APK/ZIP.',
            api,
        )

    # First fetch the already done analysis on each of the apps
    # We don't want to return this whole context back to the user
    # because its a lot of data we don't use
    # it should help the performance I guess
    first_app = deepcopy(get_context_from_db_entry(db_entry))
    second_app = deepcopy(get_context_from_db_entry(db_entry2))

    # Second, fill the common static parts that
    # are missing in the classic analysis
    for curr_app, db_context in [('first_app', first_app),
                                 ('second_app', second_app)]:

        # format informative title
        context[curr_app]['name_ver'] = '{0} - {1}'.format(
            db_context['package_name'],
            db_context['version_name'])

        # Fill all the static information
        for static_attr in static_fields:
            context[curr_app][static_attr] = db_context[static_attr]

        # Get only the subject of the cert
        subject_regex = re.compile(r'Subject: .*')
        match = subject_regex.search(
            db_context['certificate_analysis']['certificate_info'])
        if match:
            context[curr_app]['cert_subject'] = match.group()
        else:
            context[curr_app]['cert_subject'] = 'No subject'

        # Some preparations so we have some sort of same structures
        # (urls are lists inside the list which mess things up...)
        tmp_list = []
        for url_obj in db_context['urls']:
            for url in url_obj['urls']:
                # urls can mess up the table because they can be really long,
                # so let's cut them
                # escape url
                url = escape(url)
                tmp_url = url[:70]
                while len(url) > 70:
                    url = url[70:]
                    tmp_url += '<br />'
                    tmp_url += url[:70]
                tmp_list.append(tmp_url)

        db_context['urls'] = list(set(deepcopy(tmp_list)))
        tmp_list.clear()

    # apkid check - we do it here just because
    # its really ugly inside the template
    # it has a dedicated function because the result
    # is more complicated then the others...
    diff_apkid(context)

    # Third, calculate some diffs
    for section, is_tuples in [
        ('permissions', True),
        ('android_api', True),
        ('browsable_activities', True),
        ('urls', False),
    ]:
        if is_tuples:
            context[section]['common'] = [
                (x, y) for (x, y) in first_app[section].items() if x in
                second_app[section].keys()]

            # Only first
            context[section]['only_first'] = [
                (x, y) for (x, y) in first_app[section].items() if x not in
                second_app[section].keys()]

            # Only Second
            context[section]['only_second'] = [
                (x, y) for (x, y) in second_app[section].items() if x not in
                first_app[section].keys()]
        else:
            context[section]['common'] = [
                x for x in first_app[section] if x in
                second_app[section]]

            context[section]['only_first'] = [
                x for x in first_app[section] if x not in
                second_app[section]]

            context[section]['only_second'] = [
                x for x in second_app[section] if x not in
                first_app[section]]

    diff_browsable_activities(context, first_app, second_app)

    template = 'static_analysis/compare.html'
    if api:
        return context
    else:
        return render(request, template, context)



#!/usr/bin/env python3

import re

import requests

from bs4 import BeautifulSoup as Soup
from bs4 import element


ANDROID_PERMISSION_DOCS_URL = ('https://developer.android.com/'
                               'reference/android/Manifest.permission')

response = requests.get(ANDROID_PERMISSION_DOCS_URL, timeout=5)
content = Soup(response.content, 'html.parser')

online_permissions = {}

# grab all the permissions from the online docs
permission_divs = content.find_all(
    'div', {'data-version-added': re.compile(r'\d*')})
for pd in permission_divs:
    permission_name = pd.find('h3').contents[0]
    if permission_name in ['Constants', 'Manifest.permission']:
        continue
    try:
        protection_level = re.search(
            r'Protection level\: (\w+)', str(pd)).groups()[0]
    except AttributeError:
        protection_level = 'normal'
    desc = []
    for a in pd.find('p').contents:
        if type(a) is element.NavigableString:
            desc.append(str(a).strip().replace(
                '\n', '').replace('\t', '').replace('\xa0', ''))
        elif type(a) is element.Tag:
            if 'Protection level:' in a.text:
                break
            desc.append(str(a.text).strip().replace(
                '\n', '').replace('\t', '').replace('\xa0', ''))

    description = ' '.join(desc)
    online_permissions[permission_name] = [protection_level,
                                           '',
                                           description]

# check the permissions we currently have in dvm_permissions.py
DVM_PERMISSIONS = {}
eval(compile(open('../mobsf/StaticAnalyzer/views/'
                  'android/kb/dvm_permissions.py').read(),
             '<string>',
             'exec'))
MANIFEST_PERMISSIONS = DVM_PERMISSIONS['MANIFEST_PERMISSION']

for permission_name in online_permissions:
    if permission_name in MANIFEST_PERMISSIONS.keys():
        continue
    print('\'{}\': {},'.format(permission_name, str(
        online_permissions[permission_name])))



# -*- coding: utf_8 -*-
"""Handle Static Library .a file (ELF and MachO)."""
import logging
import os
from pathlib import Path

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal

from django.conf import settings
from django.shortcuts import render

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerIOS
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.ios.binary_rule_matcher import (
    binary_rule_matcher,
)
from mobsf.StaticAnalyzer.views.ios.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.StaticAnalyzer.views.ios.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    ar_extract,
    get_symbols,
    hash_gen,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import (
    MalwareDomainCheck,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)


def extract_n_get_files(checksum, src, dst):
    """Extract .a archive and get list of files."""
    files = []
    dst = Path(dst) / 'static_objects'
    dst.mkdir(parents=True, exist_ok=True)
    ar_extract(checksum, src, dst.as_posix())
    for i in dst.rglob('*.a'):
        files.append(
            os.path.relpath(dst, i.as_posix()))
    return files


def a_analysis(request, app_dict, rescan, api):
    """Independent shared library .a analysis."""
    checksum = app_dict['md5_hash']
    app_dir = Path(app_dict['app_dir'])
    app_dict['app_file'] = f'{checksum}.a'
    app_dict['app_path'] = app_dir / app_dict['app_file']
    app_dict['app_path'] = app_dict['app_path'].as_posix()
    # DB
    ipa_db = StaticAnalyzerIOS.objects.filter(MD5=checksum)
    if ipa_db.exists() and not rescan:
        context = get_context_from_db_entry(ipa_db)
    else:
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(
                request,
                'Permission Denied',
                False)
        append_scan_status(checksum, 'init')
        logger.info('Static Library Analysis Started')
        app_dict['size'] = str(
            file_size(app_dict['app_path'])) + 'MB'  # FILE SIZE
        app_dict['sha1'], app_dict['sha256'] = hash_gen(
            checksum,
            app_dict['app_path'])  # SHA1 & SHA256 HASHES
        app_dict['bin_dir'] = app_dict['app_dir']
        files = extract_n_get_files(
            checksum,
            app_dict['app_path'],
            app_dict['app_dir'],
        )
        # Get Files
        all_files = {
            'files_short': files,
            'files_long': [],
            'special_files': [],
        }
        infoplist_dict = {
            'bin_name': '',
            'bin': '',
            'id': app_dict['file_name'],
            'version': '',
            'build': '',
            'sdk': '',
            'pltfm': '',
            'min': '',
            'plist_xml': '',
            'permissions': {},
            'inseccon': {},
            'bundle_name': '',
            'build_version_name': '',
            'bundle_url_types': [],
            'bundle_supported_platforms': [],
            'bundle_version_name': '',
        }
        app_dict['infoplist'] = infoplist_dict
        app_dict['all_files'] = all_files
        app_dict['appstore'] = ''
        app_dict['secrets'] = []
        bin_dict = {
            'checksec': {},
            'libraries': [],
            'bin_code_analysis': {},
            'strings': [],
            'bin_info': {
                'endian': '',
                'bit': '',
                'arch': '',
                'subarch': '',
            },
            'bin_type': 'A',
            'framework_analysis': {},
        }
        # Analyze static library
        slib = library_analysis(
            checksum,
            app_dict['bin_dir'],
            'ar')
        bin_dict['bin_info']['arch'] = slib['ar_a']
        bin_dict['dylib_analysis'] = slib['ar_analysis']
        # Store Symbols in File Analysis
        all_files['special_files'] = get_symbols(
            slib['ar_symbols'])
        # Binary code analysis on symbols
        binary_rule_matcher(
            checksum,
            bin_dict['bin_code_analysis'],
            all_files['special_files'],
            b'')
        # Extract String metadata
        code_dict = get_strings_metadata(
            app_dict,
            bin_dict,
            all_files,
            slib['ar_strings'])
        # Domain Extraction and Malware Check
        code_dict['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_dict['urls_list'])
        logger.info('Finished URL and Email Extraction')
        # Extract Trackers from Domains
        trk = Trackers.Trackers(
            checksum,
            None,
            app_dict['tools_dir'])
        trackers = trk.get_trackers_domains_or_deps(
            code_dict['domains'], [])
        code_dict['api'] = {}
        code_dict['code_anal'] = {}
        code_dict['firebase'] = firebase_analysis(
            checksum,
            code_dict)
        code_dict['trackers'] = trackers
        context = save_get_ctx(
            app_dict,
            code_dict,
            bin_dict,
            rescan)
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(
            app_dict['app_path'])
    context['appsec'] = {}
    context['average_cvss'] = None
    template = 'static_analysis/ios_binary_analysis.html'
    if api:
        return context
    else:
        return render(request, template, context)



"""Module holding the functions for the db."""
import logging

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    get_scan_logs,
    python_list,
)
from mobsf.StaticAnalyzer.models import StaticAnalyzerWindows
from mobsf.StaticAnalyzer.models import RecentScansDB

logger = logging.getLogger(__name__)


def get_context_from_db_entry(db_entry):
    """Return the context for APPX from DB."""
    try:
        logger.info('Analysis is already Done. Fetching data from the DB...')
        context = {
            'title': 'Static Analysis',
            'version': settings.MOBSF_VER,
            'file_name': db_entry[0].FILE_NAME,
            'app_name': db_entry[0].APP_NAME,
            'publisher_name': db_entry[0].PUBLISHER_NAME,
            'size': db_entry[0].SIZE,
            'md5': db_entry[0].MD5,
            'sha1': db_entry[0].SHA1,
            'sha256': db_entry[0].SHA256,
            'app_version': db_entry[0].APP_VERSION,
            'architecture': db_entry[0].ARCHITECTURE,
            'compiler_version': db_entry[0].COMPILER_VERSION,
            'visual_studio_version': db_entry[0].VISUAL_STUDIO_VERSION,
            'visual_studio_edition': db_entry[0].VISUAL_STUDIO_EDITION,
            'target_os': db_entry[0].TARGET_OS,
            'appx_dll_version': db_entry[0].APPX_DLL_VERSION,
            'proj_guid': db_entry[0].PROJ_GUID,
            'opti_tool': db_entry[0].OPTI_TOOL,
            'target_run': db_entry[0].TARGET_RUN,
            'files': python_list(db_entry[0].FILES),
            'strings': python_list(db_entry[0].STRINGS),
            'binary_analysis': python_list(db_entry[0].BINARY_ANALYSIS),
            'binary_warnings': python_list(db_entry[0].BINARY_WARNINGS),
            'logs': get_scan_logs(db_entry[0].MD5),
        }
        return context
    except Exception:
        logger.exception('Fetching from DB')


def get_context_from_analysis(app_dic,
                              xml_dic,
                              bin_an_dic):
    """Get the context for APPX from analysis results."""
    try:
        context = {
            'title': 'Static Analysis',
            'version': settings.MOBSF_VER,
            'file_name': app_dic['app_name'],
            'app_name': bin_an_dic['bin_name'],
            'publisher_name': xml_dic['pub_name'],
            'size': app_dic['size'],
            'md5': app_dic['md5'],
            'sha1': app_dic['sha1'],
            'sha256': app_dic['sha256'],
            'app_version': xml_dic['version'],
            'architecture': xml_dic['arch'],
            'compiler_version': xml_dic['compiler_version'],
            'visual_studio_version': xml_dic['visual_studio_version'],
            'visual_studio_edition': xml_dic['visual_studio_edition'],
            'target_os': xml_dic['target_os'],
            'appx_dll_version': xml_dic['appx_dll_version'],
            'proj_guid': xml_dic['proj_guid'],
            'opti_tool': xml_dic['opti_tool'],
            'target_run': xml_dic['target_run'],
            'files': app_dic['files'],
            'strings': bin_an_dic['strings'],
            'binary_analysis': bin_an_dic['results'],
            'binary_warnings': bin_an_dic['warnings'],
            'logs': get_scan_logs(app_dic['md5']),
        }
        return context
    except Exception as exp:
        msg = 'Rendering to Template'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))


def save_or_update(update_type,
                   app_dic,
                   xml_dic,
                   bin_an_dic) -> None:
    """Save/Update an APPX DB entry."""
    try:
        values = {
            'FILE_NAME': app_dic['app_name'],
            'APP_NAME': bin_an_dic['bin_name'],
            'PUBLISHER_NAME': xml_dic['pub_name'],
            'SIZE': app_dic['size'],
            'MD5': app_dic['md5'],
            'SHA1': app_dic['sha1'],
            'SHA256': app_dic['sha256'],
            'APP_VERSION': xml_dic['version'],
            'ARCHITECTURE': xml_dic['arch'],
            'COMPILER_VERSION': xml_dic['compiler_version'],
            'VISUAL_STUDIO_VERSION': xml_dic['visual_studio_version'],
            'VISUAL_STUDIO_EDITION': xml_dic['visual_studio_edition'],
            'TARGET_OS': xml_dic['target_os'],
            'APPX_DLL_VERSION': xml_dic['appx_dll_version'],
            'PROJ_GUID': xml_dic['proj_guid'],
            'OPTI_TOOL': xml_dic['opti_tool'],
            'TARGET_RUN': xml_dic['target_run'],
            'FILES': app_dic['files'],
            'STRINGS': bin_an_dic['strings'],
            'BINARY_ANALYSIS': bin_an_dic['results'],
            'BINARY_WARNINGS': bin_an_dic['warnings'],
        }
        if update_type == 'save':
            db_entry = StaticAnalyzerWindows.objects.filter(
                MD5=app_dic['md5'])
            if not db_entry.exists():
                StaticAnalyzerWindows.objects.create(**values)
        else:
            StaticAnalyzerWindows.objects.filter(
                MD5=app_dic['md5']).update(**values)
    except Exception as exp:
        msg = 'Failed to Save/Update Database'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))
    try:
        values = {
            'APP_NAME': bin_an_dic['bin_name'],
            'PACKAGE_NAME': xml_dic['pub_name'],
            'VERSION_NAME': xml_dic['version'],
        }
        RecentScansDB.objects.filter(
            MD5=app_dic['md5']).update(**values)
    except Exception as exp:
        msg = 'Updating RecentScansDB table failed'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))



"""Common String Extraction Module."""
import logging
import shutil
import subprocess

from mobsf.StaticAnalyzer.tools.strings import (
    strings_util,
)


logger = logging.getLogger(__name__)


def get_os_strings(filename):
    try:
        strings_bin = shutil.which('strings')
        if not strings_bin:
            return None
        strings = subprocess.check_output([strings_bin, filename])
        return strings.decode('utf-8', 'ignore').splitlines()
    except Exception:
        return None


def strings_on_binary(bin_path):
    """Extract strings from binary."""
    try:
        strings = get_os_strings(bin_path)
        if strings:
            return list(set(strings))
        if isinstance(strings, list):
            return []
        # Only run if OS strings is not present
        return list(set(strings_util(bin_path)))
    except Exception:
        logger.exception('Extracting strings from binary')
    return []



import io


"""
Python Strings.

Strings-module to be independent from operating systems
Taken from: http://stackoverflow.com/a/17197027
"""


def strings_util(filename, minimum=6):
    """Print out all connected series of readable chars longer than minimum."""
    with io.open(filename, mode='rb') as f:
        result = ''
        for c in f.read().decode('utf-8', 'ignore'):
            if c in ('0123456789abcdefghijklmnopqrs'
                     'tuvwxyzABCDEFGHIJKLMNOPQRSTUV'
                     'WXYZ!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~ '):
                result += c
                continue
            if len(result) >= minimum and result[0].isalnum():
                yield '\'' + result + '\''
            result = ''



# -*- coding: utf_8 -*-
"""MobSF REST API V 1."""
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings

from mobsf.StaticAnalyzer.models import (
    RecentScansDB,
)
from mobsf.MobSF.utils import (
    get_scan_logs,
    is_md5,
)
from mobsf.MobSF.views.helpers import request_method
from mobsf.MobSF.views.home import (
    RecentScans,
    Upload,
    delete_scan,
    search,
)
from mobsf.MobSF.views.api.api_middleware import make_api_response
from mobsf.StaticAnalyzer.views.android.views import view_source
from mobsf.StaticAnalyzer.views.android.static_analyzer import static_analyzer
from mobsf.StaticAnalyzer.views.ios.views import view_source as ios_view_source
from mobsf.StaticAnalyzer.views.ios.static_analyzer import static_analyzer_ios
from mobsf.StaticAnalyzer.views.common.async_task import list_tasks
from mobsf.StaticAnalyzer.views.common.shared_func import compare_apps
from mobsf.StaticAnalyzer.views.common.suppression import (
    delete_suppression,
    list_suppressions,
    suppress_by_files,
    suppress_by_rule_id,
)
from mobsf.StaticAnalyzer.views.common.pdf import pdf
from mobsf.StaticAnalyzer.views.common.appsec import appsec_dashboard
from mobsf.StaticAnalyzer.views.windows import windows


@request_method(['POST'])
@csrf_exempt
def api_upload(request):
    """POST - Upload API."""
    upload = Upload(request)
    resp, code = upload.upload_api()
    return make_api_response(resp, code)


@request_method(['GET'])
@csrf_exempt
def api_recent_scans(request):
    """GET - get recent scans."""
    scans = RecentScans(request)
    resp = scans.recent_scans()
    if 'error' in resp:
        return make_api_response(resp, 500)
    else:
        return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_scan(request):
    """POST - Scan API."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    checksum = request.POST['hash']
    if not is_md5(checksum):
        return make_api_response(
            {'error': 'Invalid Checksum'}, 500)
    robj = RecentScansDB.objects.filter(MD5=checksum)
    if not robj.exists():
        return make_api_response(
            {'error': 'The file is not uploaded/available'}, 500)
    scan_type = robj[0].SCAN_TYPE
    # APK, Source Code (Android/iOS) ZIP, SO, JAR, AAR
    if scan_type in settings.ANDROID_EXTS:
        resp = static_analyzer(request, checksum, True)
        if 'type' in resp:
            resp = static_analyzer_ios(request, checksum, True)
        if 'error' in resp:
            response = make_api_response(resp, 500)
        else:
            response = make_api_response(resp, 200)
    # IPA
    elif scan_type in settings.IOS_EXTS:
        resp = static_analyzer_ios(request, checksum, True)
        if 'error' in resp:
            response = make_api_response(resp, 500)
        else:
            response = make_api_response(resp, 200)
    # APPX
    elif scan_type in settings.WINDOWS_EXTS:
        resp = windows.staticanalyzer_windows(request, checksum, True)
        if 'error' in resp:
            response = make_api_response(resp, 500)
        else:
            response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_scan_logs(request):
    """POST - Get Scan logs."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = get_scan_logs(request.POST['hash'])
    if not resp:
        return make_api_response(
            {'error': 'No scan logs found'}, 400)
    return make_api_response({'logs': resp}, 200)


@request_method(['POST'])
@csrf_exempt
def api_tasks(request):
    """POST - Get Scan Queue."""
    resp = list_tasks(request, True)
    if not resp:
        return make_api_response(
            {'error': 'Scan queue empty'}, 400)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_delete_scan(request):
    """POST - Delete a Scan."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = delete_scan(request, True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_pdf_report(request):
    """Generate and Download PDF."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = pdf(
        request,
        request.POST['hash'],
        api=True)
    if 'error' in resp:
        if resp.get('error') == 'Invalid scan hash':
            response = make_api_response(resp, 400)
        else:
            response = make_api_response(resp, 500)
    elif 'pdf_dat' in resp:
        response = HttpResponse(
            resp['pdf_dat'], content_type='application/pdf')
        response['Access-Control-Allow-Origin'] = '*'
    elif resp.get('report') == 'Report not Found':
        response = make_api_response(resp, 404)
    else:
        response = make_api_response(
            {'error': 'PDF Generation Error'}, 500)
    return response


@request_method(['POST'])
@csrf_exempt
def api_json_report(request):
    """Generate JSON Report."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = pdf(
        request,
        request.POST['hash'],
        api=True,
        jsonres=True)
    if 'error' in resp:
        if resp.get('error') == 'Invalid scan hash':
            response = make_api_response(resp, 400)
        else:
            response = make_api_response(resp, 500)
    elif 'report_dat' in resp:
        response = make_api_response(resp['report_dat'], 200)
    elif resp.get('report') == 'Report not Found':
        response = make_api_response(resp, 404)
    else:
        response = make_api_response(
            {'error': 'JSON Generation Error'}, 500)
    return response


@request_method(['POST'])
@csrf_exempt
def api_search(request):
    """Search by checksum or text."""
    if 'query' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = search(request, api=True)
    if 'checksum' in resp:
        request.POST = {'hash': resp['checksum']}
        return api_json_report(request)
    elif 'error' in resp:
        return make_api_response(resp, 404)


@request_method(['POST'])
@csrf_exempt
def api_view_source(request):
    """View Source for android & ios source file."""
    params = {'file', 'type', 'hash'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    if request.POST['type'] in {'eclipse', 'studio',
                                'apk', 'java', 'smali'}:
        resp = view_source.run(request, api=True)
    else:
        resp = ios_view_source.run(request, api=True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_compare(request):
    """Compare 2 apps."""
    params = {'hash1', 'hash2'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = compare_apps(
        request,
        request.POST['hash1'],
        request.POST['hash2'],
        True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_scorecard(request):
    """Generate App Score Card."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = appsec_dashboard(
        request,
        request.POST['hash'],
        api=True)
    if 'error' in resp:
        if resp.get('error') == 'Invalid scan hash':
            response = make_api_response(resp, 400)
        else:
            response = make_api_response(resp, 500)
    elif 'hash' in resp:
        response = make_api_response(resp, 200)
    elif 'not_found' in resp:
        response = make_api_response(resp, 404)
    else:
        response = make_api_response(
            {'error': 'JSON Generation Error'}, 500)
    return response


@request_method(['POST'])
@csrf_exempt
def api_suppress_by_rule_id(request):
    """POST - Suppress a rule by id."""
    params = {'rule', 'type', 'hash'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = suppress_by_rule_id(request, True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_suppress_by_files(request):
    """POST - Suppress a rule by files."""
    params = {'rule', 'hash'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = suppress_by_files(request, True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_list_suppressions(request):
    """POST - View Suppressions."""
    if 'hash' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = list_suppressions(request, True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response


@request_method(['POST'])
@csrf_exempt
def api_delete_suppression(request):
    """POST - Delete a suppression."""
    params = {'kind', 'type', 'rule', 'hash'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = delete_suppression(request, True)
    if 'error' in resp:
        response = make_api_response(resp, 500)
    else:
        response = make_api_response(resp, 200)
    return response



# -*- coding: utf_8 -*-
"""Android Dynamic Analysis."""
import logging
import os
import time
from pathlib import Path
from json import dump

from shelljob import proc


from django.http import (HttpResponseRedirect,
                         StreamingHttpResponse)
from django.conf import settings
from django.shortcuts import render
from django.db.models import ObjectDoesNotExist

from mobsf.DynamicAnalyzer.views.android.environment import (
    ANDROID_API_SUPPORTED,
    Environment,
)
from mobsf.DynamicAnalyzer.views.android.operations import (
    get_package_name,
)
from mobsf.DynamicAnalyzer.tools.webproxy import (
    get_http_tools_url,
    start_httptools_ui,
    stop_httptools,
)
from mobsf.MobSF.utils import (
    get_android_dm_exception_msg,
    get_config_loc,
    get_device,
    get_proxy_ip,
    is_md5,
    print_n_send_error_response,
    python_dict,
    python_list,
    strict_package_check,
)
from mobsf.MobSF.views.scanning import add_to_recent_scan
from mobsf.StaticAnalyzer.models import StaticAnalyzerAndroid
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)


@login_required
@permission_required(Permissions.SCAN)
def android_dynamic_analysis(request, api=False):
    """Android Dynamic Analysis Entry point."""
    try:
        scan_apps = []
        device_packages = {}
        and_ver = None
        and_sdk = None
        apks = StaticAnalyzerAndroid.objects.filter(
            APP_TYPE='apk')

        for apk in reversed(apks):

            logcat = Path(settings.UPLD_DIR) / apk.MD5 / 'logcat.txt'
            temp_dict = {
                'ICON_PATH': apk.ICON_PATH,
                'MD5': apk.MD5,
                'APP_NAME': apk.APP_NAME,
                'VERSION_NAME': apk.VERSION_NAME,
                'FILE_NAME': apk.FILE_NAME,
                'PACKAGE_NAME': apk.PACKAGE_NAME,
                'DYNAMIC_REPORT_EXISTS': logcat.exists(),
            }
            scan_apps.append(temp_dict)
        try:
            identifier = get_device()
        except Exception:
            return print_n_send_error_response(
                request, get_android_dm_exception_msg(), api)
        try:
            if identifier:
                env = Environment(identifier)
                env.connect()
                device_packages = env.get_device_packages()
                if device_packages:
                    pkg_file = Path(settings.DWD_DIR) / 'packages.json'
                    with pkg_file.open('w', encoding='utf-8') as target:
                        dump(device_packages, target)
                and_ver = env.get_android_version()
                and_sdk = env.get_android_sdk()
        except Exception:
            pass
        context = {'apps': scan_apps,
                   'identifier': identifier,
                   'android_version': and_ver,
                   'android_sdk': and_sdk,
                   'android_supported': ANDROID_API_SUPPORTED,
                   'proxy_ip': get_proxy_ip(identifier),
                   'proxy_port': settings.PROXY_PORT,
                   'settings_loc': get_config_loc(),
                   'device_packages': device_packages,
                   'title': 'MobSF Dynamic Analysis',
                   'version': settings.MOBSF_VER}
        if api:
            return context
        template = 'dynamic_analysis/android/dynamic_analysis.html'
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Dynamic Analysis')
        return print_n_send_error_response(request, exp, api)


@login_required
@permission_required(Permissions.SCAN)
def dynamic_analyzer(request, checksum, api=False):
    """Android Dynamic Analyzer Environment."""
    try:
        identifier = None
        activities = None
        deeplinks = None
        exported_activities = None
        if api:
            reinstall = request.POST.get('re_install', '1')
            install = request.POST.get('install', '1')
        else:
            reinstall = request.GET.get('re_install', '1')
            install = request.GET.get('install', '1')
        if not is_md5(checksum):
            # We need this check since checksum is not validated
            # in REST API
            return print_n_send_error_response(
                request,
                'Invalid Hash',
                api)
        package = get_package_name(checksum)
        if not package:
            return print_n_send_error_response(
                request,
                'Cannot get package name from checksum',
                api)
        logger.info('Creating Dynamic Analysis Environment for %s', package)
        try:
            identifier = get_device()
        except Exception:
            return print_n_send_error_response(
                request, get_android_dm_exception_msg(), api)

        # Get activities from the static analyzer results
        try:
            static_android_db = StaticAnalyzerAndroid.objects.get(
                MD5=checksum)
            exported_activities = python_list(
                static_android_db.EXPORTED_ACTIVITIES)
            activities = python_list(
                static_android_db.ACTIVITIES)
            deeplinks = python_dict(
                static_android_db.BROWSABLE_ACTIVITIES)
        except ObjectDoesNotExist:
            logger.warning(
                'Failed to get Activities/Deeplinks. '
                'Static Analysis not completed for the app.')
        env = Environment(identifier)
        if not env.connect_n_mount():
            msg = 'Cannot Connect to ' + identifier
            return print_n_send_error_response(request, msg, api)
        version = env.get_android_version()
        logger.info('Android Version identified as %s', version)
        xposed_first_run = False
        if not env.is_mobsfyied(version):
            msg = ('This Android instance is not MobSFyed/Outdated.\n'
                   'MobSFying the android runtime environment')
            logger.warning(msg)
            if not env.mobsfy_init():
                return print_n_send_error_response(
                    request,
                    'Failed to MobSFy the instance',
                    api)
            if version < 5:
                # Start Clipboard monitor
                env.start_clipmon()
                xposed_first_run = True
        if xposed_first_run:
            msg = ('Have you MobSFyed the instance before'
                   ' attempting Dynamic Analysis?'
                   ' Install Framework for Xposed.'
                   ' Restart the device and enable'
                   ' all Xposed modules. And finally'
                   ' restart the device once again.')
            return print_n_send_error_response(request, msg, api)
        # Clean up previous analysis
        env.dz_cleanup(checksum)
        # Configure Web Proxy
        env.configure_proxy(package, request)
        # Supported in Android 5+
        env.enable_adb_reverse_tcp(version)
        # Apply Global Proxy to device
        env.set_global_proxy(version)
        if install == '1':
            # Install APK
            apk_path = Path(settings.UPLD_DIR) / checksum / f'{checksum}.apk'
            status, output = env.install_apk(
                apk_path.as_posix(),
                package,
                reinstall)
            if not status:
                # Unset Proxy
                env.unset_global_proxy()
                msg = (f'This APK cannot be installed. Is this APK '
                       f'compatible the Android VM/Emulator?\n{output}')
                return print_n_send_error_response(
                    request,
                    msg,
                    api)
        logger.info('Testing Environment is Ready!')
        context = {'package': package,
                   'hash': checksum,
                   'android_version': version,
                   'version': settings.MOBSF_VER,
                   'activities': activities,
                   'exported_activities': exported_activities,
                   'deeplinks': deeplinks,
                   'title': 'Dynamic Analyzer'}
        template = 'dynamic_analysis/android/dynamic_analyzer.html'
        if api:
            return context
        return render(request, template, context)
    except Exception:
        logger.exception('Dynamic Analyzer')
        return print_n_send_error_response(
            request,
            'Dynamic Analysis Failed.',
            api)


@login_required
@permission_required(Permissions.SCAN)
def httptools_start(request):
    """Start httprools UI."""
    logger.info('Starting httptools Web UI')
    try:
        httptools_url = get_http_tools_url(request)
        stop_httptools(httptools_url)
        start_httptools_ui(settings.PROXY_PORT)
        time.sleep(3)
        logger.info('httptools UI started')
        if request.GET['project']:
            project = request.GET['project']
        else:
            project = ''
        url = f'{httptools_url}/dashboard/{project}'
        return HttpResponseRedirect(url)
    except Exception:
        logger.exception('Starting httptools Web UI')
        err = 'Error Starting httptools UI'
        return print_n_send_error_response(request, err)


@login_required
@permission_required(Permissions.SCAN)
def logcat(request, api=False):
    logger.info('Starting Logcat streaming')
    try:
        pkg = request.GET.get('package')
        if pkg:
            if not strict_package_check(pkg):
                return print_n_send_error_response(
                    request,
                    'Invalid package name',
                    api)
            template = 'dynamic_analysis/android/logcat.html'
            return render(request, template, {'package': pkg})
        if api:
            app_pkg = request.POST['package']
        else:
            app_pkg = request.GET.get('app_package')
        if app_pkg:
            if not strict_package_check(app_pkg):
                return print_n_send_error_response(
                    request,
                    'Invalid package name',
                    api)
            adb = os.environ['MOBSF_ADB']
            g = proc.Group()
            g.run([adb, 'logcat', app_pkg + ':V', '*:*'])

            def read_process():
                while g.is_pending():
                    lines = g.readlines()
                    for _, line in lines:
                        yield 'data:{}\n\n'.format(line)
            return StreamingHttpResponse(read_process(),
                                         content_type='text/event-stream')
        return print_n_send_error_response(
            request,
            'Invalid parameters',
            api)
    except Exception:
        logger.exception('Logcat Streaming')
        err = 'Error in Logcat streaming'
        return print_n_send_error_response(request, err, api)


@login_required
@permission_required(Permissions.SCAN)
def trigger_static_analysis(request, checksum):
    """On device APK Static Analysis."""
    try:
        identifier = None
        if not is_md5(checksum):
            return print_n_send_error_response(
                request,
                'Invalid MD5')
        package = get_package_name(checksum)
        if not package:
            return print_n_send_error_response(
                request,
                'Cannot get package name from checksum')
        try:
            identifier = get_device()
        except Exception:
            err = 'Cannot connect to Android Runtime'
            return print_n_send_error_response(request, err)
        env = Environment(identifier)
        scan_type = env.get_apk(checksum, package)
        if not scan_type:
            err = 'Failed to download APK file'
            return print_n_send_error_response(request, err)
        file_name = f'{package}.apk'
        if scan_type == 'apks':
            file_name = f'{file_name}s'
        data = {
            'analyzer': 'static_analyzer',
            'status': 'success',
            'hash': checksum,
            'scan_type': scan_type,
            'file_name': file_name,
        }
        add_to_recent_scan(data)
        return HttpResponseRedirect(f'/static_analyzer/{checksum}/')
    except Exception:
        msg = 'On device APK Static Analysis'
        logger.exception(msg)
        return print_n_send_error_response(request, msg)



"""Common Utils."""
import ast
import base64
import hashlib
import io
import json
import logging
import ntpath
import os
import platform
import random
import re
import sys
import shutil
import signal
import string
import subprocess
import stat
import sqlite3
import unicodedata
import threading
from pathlib import Path
from concurrent.futures import (
    ThreadPoolExecutor,
    TimeoutError as ThreadPoolTimeoutError,
)

from packaging.version import Version

import distro

import psutil

import requests

from django.shortcuts import render
from django.utils import timezone

from mobsf.StaticAnalyzer.models import RecentScansDB
from mobsf.MobSF. init import api_key

from . import settings


logger = logging.getLogger(__name__)
ADB_PATH = None
BASE64_REGEX = re.compile(r'^[-A-Za-z0-9+/]*={0,3}$')
MD5_REGEX = re.compile(r'^[0-9a-f]{32}$')
# Regex to capture strings between quotes or <string> tag
STRINGS_REGEX = re.compile(r'(?<=\")(.+?)(?=\")|(?<=\<string>)(.+?)(?=\<)')
# MobSF Custom regex to catch maximum URI like strings
URL_REGEX = re.compile(
    (
        r'((?:https?://|s?ftps?://|'
        r'file://|javascript:|data:|www\d{0,3}[.])'
        r'[\w().=/;,#:@?&~*+!$%\'{}-]+)'
    ),
    re.UNICODE | re.IGNORECASE)
EMAIL_REGEX = re.compile(r'[\w+.-]{1,20}@[\w-]{1,20}\.[\w]{2,10}')
USERNAME_REGEX = re.compile(r'^\w[\w\-\@\.]{1,35}$')
GOOGLE_API_KEY_REGEX = re.compile(r'AIza[0-9A-Za-z-_]{35}$')
GOOGLE_APP_ID_REGEX = re.compile(r'\d{1,2}:\d{1,50}:android:[a-f0-9]{1,50}')
PKG_REGEX = re.compile(
    r'package\s+([a-zA-Z_][a-zA-Z0-9_]*(\.[a-zA-Z_][a-zA-Z0-9_]*)*);')
IOS_DEVICE_ID_REGEX = re.compile(r'^[a-fA-F0-9-]{20,40}$')
SSH_DEVICE_ID_REGEX = re.compile(
    r'^(?:\[(?P<ipv6>[^\]]+)\]|(?P<host>[^:\[\]]+)):(?P<port>\d{1,5})$')


class Color(object):
    GREEN = '\033[92m'
    GREY = '\033[0;37m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    END = '\033[0m'


def upstream_proxy(flaw_type):
    """Set upstream Proxy if needed."""
    if settings.UPSTREAM_PROXY_ENABLED:
        if not settings.UPSTREAM_PROXY_USERNAME:
            proxy_port = str(settings.UPSTREAM_PROXY_PORT)
            proxy_host = '{}://{}:{}'.format(
                settings.UPSTREAM_PROXY_TYPE,
                docker_translate_proxy_ip(settings.UPSTREAM_PROXY_IP),
                proxy_port)
            proxies = {flaw_type: proxy_host}
        else:
            proxy_port = str(settings.UPSTREAM_PROXY_PORT)
            proxy_host = '{}://{}:{}@{}:{}'.format(
                settings.UPSTREAM_PROXY_TYPE,
                settings.UPSTREAM_PROXY_USERNAME,
                settings.UPSTREAM_PROXY_PASSWORD,
                docker_translate_proxy_ip(settings.UPSTREAM_PROXY_IP),
                proxy_port)
            proxies = {flaw_type: proxy_host}
    else:
        proxies = {flaw_type: None}
    verify = settings.UPSTREAM_PROXY_SSL_VERIFY in ('1', '"1"')
    return proxies, verify


def get_system_resources():
    """Get CPU and Memory Available."""
    # Get number of physical cores
    physical_cores = psutil.cpu_count(logical=False)
    # Get number of logical processors (threads)
    logical_processors = psutil.cpu_count(logical=True)
    # Get total RAM
    total_ram = psutil.virtual_memory().total / (1024 ** 3)  # Convert bytes to GB
    return physical_cores, logical_processors, total_ram


def print_version():
    """Print MobSF Version."""
    logger.info(settings.BANNER)
    ver = settings.MOBSF_VER
    logger.info('Author: Ajin Abraham | opensecurity.in')
    mobsf_api_key = api_key(settings.MOBSF_HOME)
    if platform.system() == 'Windows':
        logger.info('Mobile Security Framework %s', ver)
        print(f'REST API Key: {mobsf_api_key}')
        print('Default Credentials: mobsf/mobsf')
    else:
        logger.info(
            '%sMobile Security Framework %s%s', Color.GREY, ver, Color.END)
        print(f'REST API Key: {Color.BOLD}{mobsf_api_key}{Color.END}')
        print(f'Default Credentials: {Color.BOLD}mobsf/mobsf{Color.END}')
    os = platform.system()
    pltfm = platform.platform()
    dist = ' '.join(distro.linux_distribution(
        full_distribution_name=False)).strip()
    dst_str = ' '
    if dist:
        dst_str = f' ({dist}) '
    env_str = f'OS Environment: {os}{dst_str}{pltfm}'
    logger.info(env_str)
    python_version = sys.version.split(' ')[0]
    logger.info('Python Version: %s', python_version)
    cores, threads, ram = get_system_resources()
    logger.info('CPU Cores: %s, Threads: %s, RAM: %.2f GB', cores, threads, ram)
    find_java_binary()
    check_basic_env()
    thread = threading.Thread(target=check_update, name='check_update')
    thread.start()


def check_update():
    try:
        if not is_internet_available():
            logger.warning('Internet Not Available. Skipping Update check')
            return
        logger.info('Checking for Update.')
        github_url = settings.GITHUB_URL
        try:
            proxies, verify = upstream_proxy('https')
        except Exception:
            logger.exception('Setting upstream proxy')
        local_version = settings.VERSION
        response = requests.head(github_url, timeout=5,
                                 proxies=proxies, verify=verify)
        remote_version = response.next.path_url.split('v')[1]
        if remote_version:
            sem_loc = Version(local_version)
            sem_rem = Version(remote_version)
            if sem_loc < sem_rem:
                logger.warning('A new version of MobSF is available, '
                               'Please update to %s from master branch.',
                               remote_version)
            else:
                logger.info('No updates available.')
    except requests.exceptions.HTTPError:
        logger.warning('\nCannot check for updates..'
                       ' No Internet Connection Found.')
        return
    except Exception:
        logger.exception('Cannot Check for updates.')


def find_java_binary():
    """Find Java."""
    # Respect user settings
    if platform.system() == 'Windows':
        jbin = 'java.exe'
    else:
        jbin = 'java'
    if is_dir_exists(settings.JAVA_DIRECTORY):
        if settings.JAVA_DIRECTORY.endswith('/'):
            return settings.JAVA_DIRECTORY + jbin
        elif settings.JAVA_DIRECTORY.endswith('\\'):
            return settings.JAVA_DIRECTORY + jbin
        else:
            return settings.JAVA_DIRECTORY + '/' + jbin
    if os.getenv('JAVA_HOME'):
        java = os.path.join(
            os.getenv('JAVA_HOME'),
            'bin',
            jbin)
        if is_file_exists(java):
            return java
    return 'java'


def find_aapt(tool_name):
    """Find the specified tool (aapt or aapt2)."""
    # Check system PATH for the tool
    tool_path = shutil.which(tool_name)
    if tool_path:
        return tool_path

    # Check common Android SDK locations
    home_dir = Path.home()  # Get the user's home directory
    sdk_paths = [
        home_dir / 'Library' / 'Android' / 'sdk',  # macOS
        home_dir / 'Android' / 'Sdk',              # Linux
        home_dir / 'AppData' / 'Local' / 'Android' / 'Sdk',  # Windows
    ]

    for sdk_path in sdk_paths:
        build_tools_path = sdk_path / 'build-tools'
        if build_tools_path.exists():
            for version in sorted(build_tools_path.iterdir(), reverse=True):
                tool_path = version / tool_name
                if tool_path.exists():
                    return str(tool_path)

    return None


def print_n_send_error_response(request,
                                msg,
                                api=False,
                                exp='Description'):
    """Print and log errors."""
    logger.error(msg)
    if api:
        api_response = {'error': msg}
        return api_response
    else:
        context = {
            'title': 'Error',
            'exp': exp,
            'doc': msg,
            'version': settings.MOBSF_VER,
        }
        template = 'general/error.html'
        return render(request, template, context, status=500)


def filename_from_path(path):
    head, tail = ntpath.split(path)
    return tail or ntpath.basename(head)


def get_md5(data):
    if isinstance(data, str):
        data = data.encode('utf-8')
    return hashlib.md5(data).hexdigest()


def find_between(s, first, last):
    try:
        start = s.index(first) + len(first)
        end = s.index(last, start)
        return s[start:end]
    except ValueError:
        return ''


def is_number(s):
    if not s:
        return False
    if s == 'NaN':
        return False
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def python_list(value):
    if not value:
        value = []
    if isinstance(value, list):
        return value
    return ast.literal_eval(value)


def python_dict(value):
    if not value:
        value = {}
    if isinstance(value, dict):
        return value
    return ast.literal_eval(value)


def is_base64(b_str):
    return BASE64_REGEX.match(b_str)


def is_internet_available():
    try:
        proxies, verify = upstream_proxy('https')
    except Exception:
        logger.exception('Setting upstream proxy')
    try:
        requests.get(settings.GOOGLE,
                     timeout=5,
                     proxies=proxies,
                     verify=verify)
        return True
    except Exception:
        try:
            requests.get(settings.BAIDU,
                         timeout=5,
                         proxies=proxies,
                         verify=verify)
            return True
        except Exception:
            return False


def sha256(file_path):
    blocksize = 65536
    hasher = hashlib.sha256()
    with io.open(file_path, mode='rb') as afile:
        buf = afile.read(blocksize)
        while buf:
            hasher.update(buf)
            buf = afile.read(blocksize)
    return hasher.hexdigest()


def sha256_object(file_obj):
    blocksize = 65536
    hasher = hashlib.sha256()
    buf = file_obj.read(blocksize)
    while buf:
        hasher.update(buf)
        buf = file_obj.read(blocksize)
    return hasher.hexdigest()


def gen_sha256_hash(msg):
    """Generate SHA 256 Hash of the message."""
    if isinstance(msg, str):
        msg = msg.encode('utf-8')
    hash_object = hashlib.sha256(msg)
    return hash_object.hexdigest()


def is_file_exists(file_path):
    if os.path.isfile(file_path):
        return True
    # This fix situation where a user just typed "adb" or another executable
    # inside settings.py/config.py
    if shutil.which(file_path):
        return True
    else:
        return False


def is_dir_exists(dir_path):
    if os.path.isdir(dir_path):
        return True
    else:
        return False


def find_process_by(name):
    """Return a set of process path matching name."""
    proc = set()
    for p in psutil.process_iter(attrs=['name']):
        if (name == p.info['name']):
            proc.add(p.exe())
    return proc


def docker_translate_localhost(identifier):
    """Convert localhost to host.docker.internal."""
    if not identifier:
        return identifier
    if not os.getenv('MOBSF_PLATFORM') == 'docker':
        return identifier
    try:
        identifier = identifier.strip()
        docker_internal = 'host.docker.internal:'
        if re.match(r'^emulator-\d{4}$', identifier):
            adb_port = int(identifier.split('emulator-')[1]) + 1
            # ADB port is console port + 1
            return f'{docker_internal}{adb_port}'
        m = re.match(r'^(localhost|127\.0\.0\.1):\d{1,5}$', identifier)
        if m:
            adb_port = int(identifier.split(m.group(1))[1].replace(':', ''))
            return f'{docker_internal}{adb_port}'
        return identifier
    except Exception:
        logger.exception('Failed to convert device '
                         'identifier for docker connectivity')
        return identifier


def docker_translate_proxy_ip(ip):
    """Convert localhost proxy ip to host.docker.internal."""
    if not os.getenv('MOBSF_PLATFORM') == 'docker':
        return ip
    if ip and ip.strip() in ('127.0.0.1', 'localhost'):
        return 'host.docker.internal'
    return ip


def get_device():
    """Get Device."""
    if os.getenv('ANALYZER_IDENTIFIER'):
        return docker_translate_localhost(
            os.getenv('ANALYZER_IDENTIFIER'))
    elif settings.ANALYZER_IDENTIFIER:
        return docker_translate_localhost(
            settings.ANALYZER_IDENTIFIER)
    else:
        dev_id = ''
        out = subprocess.check_output([get_adb(), 'devices']).splitlines()
        if len(out) > 2:
            dev_id = out[1].decode('utf-8').split('\t')[0]
            if 'daemon started successfully' not in dev_id:
                return docker_translate_localhost(dev_id)
    logger.error(get_android_dm_exception_msg())


def get_adb():
    """Get ADB binary path."""
    try:
        adb_loc = None
        adb_msg = ('Set adb path, ADB_BINARY in'
                   f' {get_config_loc()}'
                   ' with same adb binary used'
                   ' by Genymotion VM/Emulator AVD.')
        global ADB_PATH
        if (len(settings.ADB_BINARY) > 0
                and is_file_exists(settings.ADB_BINARY)):
            ADB_PATH = settings.ADB_BINARY
            return ADB_PATH
        if ADB_PATH:
            return ADB_PATH
        if platform.system() == 'Windows':
            adb_loc = find_process_by('adb.exe')
        else:
            adb_loc = find_process_by('adb')
        if len(adb_loc) > 1:
            logger.warning('Multiple ADB locations found. %s', adb_msg)
            logger.warning(adb_loc)
        if adb_loc:
            ADB_PATH = adb_loc.pop()
            return ADB_PATH
    except Exception:
        if not adb_loc:
            logger.warning('Cannot find adb! %s', adb_msg)
        logger.exception('Getting ADB Location')
    finally:
        if ADB_PATH:
            os.environ['MOBSF_ADB'] = ADB_PATH
        else:
            os.environ['MOBSF_ADB'] = 'adb'
            logger.warning('Dynamic Analysis related '
                           'functions will not work. '
                           '\nMake sure a Genymotion Android VM/'
                           'Android Studio Emulator'
                           ' is running before performing'
                           ' Dynamic Analysis.')
    return 'adb'


def check_basic_env():
    """Check if we have basic env for MobSF to run."""
    logger.info('MobSF Basic Environment Check')
    try:
        import http_tools  # noqa F401
    except ImportError:
        logger.exception('httptools not installed!')
        os.kill(os.getpid(), signal.SIGTERM)
    try:
        import lxml  # noqa F401
    except ImportError:
        logger.exception('lxml is not installed!')
        os.kill(os.getpid(), signal.SIGTERM)
    if not is_file_exists(find_java_binary()):
        logger.error(
            'JDK 8+ is not available. '
            'Set JAVA_HOME environment variable'
            ' or JAVA_DIRECTORY in '
            '%s', get_config_loc())
        logger.info('Current Configuration: '
                    'JAVA_DIRECTORY=%s', settings.JAVA_DIRECTORY)
        logger.info('Example Configuration:'
                    '\nJAVA_DIRECTORY = "C:/Program Files/'
                    'Java/jdk1.7.0_17/bin/"'
                    '\nJAVA_DIRECTORY = "/usr/bin/"')
        os.kill(os.getpid(), signal.SIGTERM)


def update_local_db(db_name, url, local_file):
    """Update Local DBs."""
    update = None
    inmemoryfile = None
    try:
        proxies, verify = upstream_proxy('https')
    except Exception:
        logger.exception('[ERROR] Setting upstream proxy')
    try:
        response = requests.get(url,
                                timeout=3,
                                proxies=proxies,
                                verify=verify)
        resp = response.content
        inmemoryfile = io.BytesIO(resp)
        # Create on first run
        if not is_file_exists(local_file):
            return resp
        # Check1: SHA256 Change
        if sha256_object(inmemoryfile) != sha256(local_file):
            # Hash Changed
            logger.info('%s Database is outdated!', db_name)
            update = resp
        else:
            logger.info('%s Database is up-to-date', db_name)
        return update
    except (requests.exceptions.ReadTimeout,
            requests.exceptions.ConnectionError):
        logger.warning('Failed to download %s DB.', db_name)
    except Exception:
        logger.exception('[ERROR] %s DB Update', db_name)
        return update
    finally:
        if inmemoryfile:
            inmemoryfile.truncate(0)


def read_sqlite(sqlite_file):
    """Sqlite Dump - Readable Text."""
    logger.info('Reading SQLite db')
    table_dict = {}
    try:
        con = sqlite3.connect(f'file:{sqlite_file}?mode=ro', uri=True)
        cur = con.cursor()
        cur.execute('SELECT name FROM sqlite_master WHERE type=\'table\';')
        tables = cur.fetchall()
        for table in tables:
            table_name = table[0]
            safe_name = table_name.replace('"', '""')
            table_dict[table_name] = {'head': [], 'data': []}
            cur.execute(f'PRAGMA table_info("{safe_name}")')
            rows = cur.fetchall()
            for sq_row in rows:
                table_dict[table_name]['head'].append(sq_row[1])
            cur.execute(f'SELECT * FROM "{safe_name}"')
            rows = cur.fetchall()
            for sq_row in rows:
                tmp_row = []
                for each_row in sq_row:
                    tmp_row.append(str(each_row))
                table_dict[table_name]['data'].append(tmp_row)
    except Exception:
        logger.exception('Reading SQLite db')
    return table_dict


def get_network():
    """Get Network IPs."""
    ips = []
    try:
        for det in psutil.net_if_addrs().values():
            ips.append(det[0].address)
    except Exception:
        logger.exception('Failed to enumerate network interfaces')
    return ips


def get_proxy_ip(identifier):
    """Get Proxy IP."""
    proxy_ip = None
    try:
        if not identifier:
            return proxy_ip
        ips = get_network()
        if ':' not in identifier or not ips:
            return proxy_ip
        device_ip = identifier.split(':', 1)[0]
        ip_range = device_ip.rsplit('.', 1)[0]
        guess_ip = ip_range + '.1'
        if guess_ip in ips:
            return guess_ip
        for ip_addr in ips:
            to_check = ip_addr.rsplit('.', 1)[0]
            if to_check == ip_range:
                return ip_addr
    except Exception:
        logger.error('Error getting Proxy IP')
    return proxy_ip


def file_size(app_path):
    """Return the size of the file."""
    return round(float(os.path.getsize(app_path)) / (1024 * 1024), 2)


def is_md5(user_input):
    """Check if string is valid MD5."""
    stat = MD5_REGEX.match(user_input)
    if not stat:
        logger.error('Invalid scan hash')
    return stat


def get_config_loc():
    """Get config location."""
    if settings.USE_HOME:
        return os.path.join(
            os.path.expanduser('~'),
            '.MobSF',
            'config.py',
        )
    else:
        return 'MobSF/settings.py'


def strict_package_check(user_input):
    """Strict package name check.

    For android package and ios bundle id
    """
    pat = re.compile(r'^([a-zA-Z]{1}[\w.-]{1,255})$')
    resp = re.match(pat, user_input)
    if not resp or '..' in user_input:
        logger.error('Invalid package name/bundle id/class name')
    return resp


def strict_ios_class(user_input):
    """Strict check to see if input is valid iOS class."""
    pat = re.compile(r'^([\w\.]+)$')
    resp = re.match(pat, user_input)
    if not resp:
        logger.error('Invalid class name')
    return resp


def is_instance_id(user_input):
    """Check if string is valid instance id."""
    reg = r'^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'
    stat = re.match(reg, user_input)
    if not stat:
        logger.error('Invalid instance identifier')
    return stat


def common_check(instance_id):
    """Common checks for instance APIs."""
    if not getattr(settings, 'CORELLIUM_API_KEY', ''):
        return {
            'status': 'failed',
            'message': 'Missing Corellium API key'}
    elif not is_instance_id(instance_id):
        return {
            'status': 'failed',
            'message': 'Invalid instance identifier'}
    else:
        return None


def is_zip_magic(file_obj):
    magic = file_obj.read(4)
    file_obj.seek(0, 0)
    # ZIP magic PK.. no support for spanned and empty arch
    return bool(magic == b'\x50\x4B\x03\x04')


def is_elf_so_magic(file_obj):
    magic = file_obj.read(4)
    file_obj.seek(0, 0)
    # ELF/SO Magic
    return bool(magic == b'\x7F\x45\x4C\x46')


def is_dylib_magic(file_obj):
    magic = file_obj.read(4)
    file_obj.seek(0, 0)
    # DYLIB Magic
    magics = (
        b'\xCA\xFE\xBA\xBE',  # 32 bit
        b'\xFE\xED\xFA\xCE',  # 32 bit
        b'\xCE\xFA\xED\xFE',  # 32 bit
        b'\xFE\xED\xFA\xCF',  # 64 bit
        b'\xCF\xFA\xED\xFE',  # 64 bit
        b'\xCA\xFE\xBA\xBF',  # 64 bit
    )
    return bool(magic in magics)


def is_a_magic(file_obj):
    magic = file_obj.read(4)
    file_obj.seek(0, 0)
    magics = (
        b'\x21\x3C\x61\x72',
        b'\xCA\xFE\xBA\xBF',  # 64 bit
        b'\xCA\xFE\xBA\xBE',  # 32 bit
    )
    return bool(magic in magics)


def disable_print():
    sys.stdout = open(os.devnull, 'w')


# Restore
def enable_print():
    sys.stdout = sys.__stdout__


def find_key_in_dict(key, var):
    """Recursively look up a key in a nested dict."""
    if hasattr(var, 'items'):
        for k, v in var.items():
            if k == key:
                yield v
            if isinstance(v, dict):
                for result in find_key_in_dict(key, v):
                    yield result
            elif isinstance(v, list):
                for d in v:
                    for result in find_key_in_dict(key, d):
                        yield result


def key(data, key_name):
    """Return the data for a key_name."""
    return data.get(key_name)


def replace(value, arg):
    """
    Replacing filter.

    Use `{{ "aaa"|replace:"a|b" }}`
    """
    if len(arg.split('|')) != 2:
        return value

    what, to = arg.split('|')
    return value.replace(what, to)


def pathify(value):
    """Convert to path."""
    return value.replace('.', '/')


def relative_path(value):
    """Show relative path to two parents."""
    sep = None
    if '/' in value:
        sep = '/'
    elif '\\\\' in value:
        sep = '\\\\'
    elif '\\' in value:
        sep = '\\'
    if not sep or value.count(sep) < 2:
        return value
    path = Path(value)
    return path.relative_to(path.parent.parent).as_posix()


def pretty_json(value):
    """Pretty print JSON."""
    try:
        return json.dumps(json.loads(value), indent=4)
    except Exception:
        return value


def base64_decode(value):
    """Try Base64 decode."""
    commonb64s = ('eyJ0')
    decoded = None
    try:
        if is_base64(value) or value.startswith(commonb64s):
            decoded = base64.b64decode(
                value).decode('ISO-8859-1')
    except Exception:
        pass
    if decoded:
        return f'{value}\n\nBase64 Decoded: {decoded}'
    return value


def base64_encode(value):
    """Base64 encode."""
    if isinstance(value, str):
        value = value.encode('utf-8')
    return base64.b64encode(value)


def android_component(data):
    """Return Android component from data."""
    cmp = ''
    if 'Activity-Alias' in data:
        cmp = 'activity_alias_'
    elif 'Activity' in data:
        cmp = 'activity_'
    elif 'Service' in data:
        cmp = 'service_'
    elif 'Content Provider' in data:
        cmp = 'provider_'
    elif 'Broadcast Receiver' in data:
        cmp = 'receiver_'
    return cmp


def get_android_dm_exception_msg():
    return (
        'Is your Android VM/emulator running? MobSF cannot'
        ' find the android device identifier.'
        ' Please read official documentation.'
        ' If this error persists, set ANALYZER_IDENTIFIER in '
        f'{get_config_loc()} or via environment variable'
        ' MOBSF_ANALYZER_IDENTIFIER')


def get_android_src_dir(app_dir, typ):
    """Get Android source code location."""
    src = None
    if typ == 'apk':
        src = app_dir / 'java_source'
    elif typ == 'studio':
        src = app_dir / 'app' / 'src' / 'main' / 'java'
        kt = app_dir / 'app' / 'src' / 'main' / 'kotlin'
        if not src.exists() and kt.exists():
            src = kt
    elif typ == 'eclipse':
        src = app_dir / 'src'
    return src


def settings_enabled(attr):
    """Get settings state if present."""
    disabled = ('', ' ', '""', '" "', '0', '"0"', False)
    try:
        return getattr(settings, attr) not in disabled
    except Exception:
        return False


def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
    """Generate random string."""
    return ''.join(random.choice(chars) for _ in range(size))


def append_scan_status(checksum, status, exception=None):
    """Append Scan Status to Database."""
    try:
        db_obj = RecentScansDB.objects.get(MD5=checksum)
        if status == 'init':
            db_obj.SCAN_LOGS = []
            db_obj.save()
            return
        current_logs = python_dict(db_obj.SCAN_LOGS)
        current_logs.append({
            'timestamp': timezone.now().strftime('%Y-%m-%d %H:%M:%S'),
            'status': status,
            'exception': exception})
        db_obj.SCAN_LOGS = current_logs
        db_obj.save()
    except RecentScansDB.DoesNotExist:
        # Expected to fail for iOS Dynamic Analysis Report Generation
        # Calls MalwareScan and TrackerScan with different checksum
        pass
    except Exception:
        logger.exception('Appending Scan Status to Database')


def get_scan_logs(checksum):
    """Get the scan logs for the given checksum."""
    try:
        db_entry = RecentScansDB.objects.filter(MD5=checksum)
        if db_entry.exists():
            return python_list(db_entry[0].SCAN_LOGS)
    except Exception:
        msg = 'Fetching scan logs from the DB failed.'
        logger.exception(msg)
    return []


class TaskTimeoutError(Exception):
    pass


def run_with_timeout(func, limit, *args, **kwargs):
    with ThreadPoolExecutor(max_workers=1) as executor:
        future = executor.submit(func, *args, **kwargs)
        try:
            return future.result(timeout=limit)
        except ThreadPoolTimeoutError:
            msg = f'function <{func.__name__}> timed out after {limit} seconds'
            raise TaskTimeoutError(msg)


def set_permissions(path):
    base_path = Path(path)
    # Read/Write for directories without execute
    perm_dir = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH
    # Read/Write for files
    perm_file = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH

    # Set permissions for directories and files
    for item in base_path.rglob('*'):
        try:
            if item.is_dir():
                item.chmod(perm_dir)
            elif item.is_file():
                item.chmod(perm_file)
        except Exception:
            pass


def parse_host_port(text):
    # Match [IPv6]:port OR host:port
    ipv6_match = re.match(r'^\[(.+)\]:(\d+)$', text)
    if ipv6_match:
        return ipv6_match.group(1), int(ipv6_match.group(2))

    ipv4_match = re.match(r'^([^:]+):(\d+)$', text)
    if ipv4_match:
        return ipv4_match.group(1), int(ipv4_match.group(2))

    raise ValueError('Input must be in format host:port or [IPv6]:port')



# -*- coding: utf_8 -*-
"""iOS View Source."""
import io
import json
import logging
import ntpath
import os
import plistlib
from pathlib import Path

from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.utils.html import escape

from mobsf.MobSF.forms import FormUtil
from mobsf.MobSF.security import is_safe_path
from mobsf.MobSF.utils import (
    is_file_exists,
    print_n_send_error_response,
    read_sqlite,
)
from mobsf.StaticAnalyzer.forms import (
    ViewSourceIOSApiForm,
    ViewSourceIOSForm,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)

logger = logging.getLogger(__name__)


def set_ext_api(file_path):
    """Smart Function to set Extension."""
    ext = file_path.split('.')[-1]
    if ext == 'plist':
        return 'plist'
    elif ext == 'xml':
        return 'xml'
    elif ext in ['sqlitedb', 'db', 'sqlite']:
        return 'db'
    elif ext == 'm':
        return 'm'
    else:
        return 'txt'


@login_required
def run(request, api=False):
    """View iOS Files."""
    try:
        logger.info('View iOS Source File')
        exp = 'Error Description'
        file_format = None
        if api:
            fil = request.POST['file']
            md5_hash = request.POST['hash']
            mode = request.POST['type']
            viewsource_form = ViewSourceIOSApiForm(request.POST)
        else:
            fil = request.GET['file']
            md5_hash = request.GET['md5']
            mode = request.GET['type']
            viewsource_form = ViewSourceIOSForm(request.GET)
        typ = set_ext_api(fil)
        if not viewsource_form.is_valid():
            err = FormUtil.errors_message(viewsource_form)
            if api:
                return err
            return print_n_send_error_response(request, err, False, exp)
        base = Path(settings.UPLD_DIR) / md5_hash
        if mode == 'ipa':
            src1 = base / 'payload'
            src2 = base / 'Payload'
            if src1.exists():
                src = src1
            elif src2.exists():
                src = src2
            else:
                raise Exception('MobSF cannot find Payload directory')
        elif mode == 'ios':
            src = base
        sfile = src / fil
        sfile = sfile.as_posix()
        if not is_safe_path(src, sfile, fil):
            msg = 'Path Traversal Detected!'
            if api:
                return {'error': 'Path Traversal Detected!'}
            return print_n_send_error_response(request, msg, False, exp)
        dat = ''
        sql_dump = {}
        if typ == 'm':
            file_format = 'cpp'
            with io.open(sfile,
                         mode='r',
                         encoding='utf8',
                         errors='ignore') as flip:
                dat = flip.read()
        elif typ == 'xml':
            file_format = 'xml'
            with io.open(sfile,
                         mode='r',
                         encoding='utf8',
                         errors='ignore') as flip:
                dat = flip.read()
        elif typ == 'plist':
            file_format = 'json'
            try:
                with open(sfile, 'rb') as f:
                    # Attempt to load the plist, binary or XML
                    dat = plistlib.load(f)
                    # Convert the plist data to JSON for output
                    dat = json.dumps(dat, indent=4, sort_keys=True)
            except plistlib.InvalidFileException:
                # Handle invalid plist files (e.g., if it isn't binary or XML)
                file_format = 'xml'
                dat = Path(sfile).read_text()
            except Exception:
                dat = None
        elif typ == 'db':
            file_format = 'asciidoc'
            sql_dump = read_sqlite(sfile)
        elif typ == 'txt' and fil == 'classdump.txt':
            file_format = 'cpp'
            app_dir = os.path.join(settings.UPLD_DIR, md5_hash + '/')
            cls_dump_file = os.path.join(app_dir, 'classdump.txt')
            if is_file_exists(cls_dump_file):
                with io.open(cls_dump_file,  # lgtm [py/path-injection]
                             mode='r',
                             encoding='utf8',
                             errors='ignore') as flip:
                    dat = flip.read()
            else:
                dat = 'Class Dump result not Found'
        elif typ == 'txt':
            file_format = 'text'
            with io.open(sfile,
                         mode='r',
                         encoding='utf8',
                         errors='ignore') as flip:
                dat = flip.read()
        else:
            if api:
                return {'error': 'Invalid Parameters'}
            return HttpResponseRedirect('/error/')
        context = {
            'title': escape(ntpath.basename(fil)),
            'file': escape(ntpath.basename(fil)),
            'type': file_format,
            'data': dat,
            'sqlite': sql_dump,
            'version': settings.MOBSF_VER,
        }
        template = 'general/view.html'
        if api:
            return context
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Error Viewing Source')
        msg = str(exp)
        exp = exp.__doc__
        if api:
            return print_n_send_error_response(request, msg, True, exp)
        return print_n_send_error_response(request, msg, False, exp)



# -*- coding: utf_8 -*-
"""
Shared Functions.

Module providing the shared functions for iOS and Android
"""
import hashlib
import logging
import os
import platform
import re
import shutil
import subprocess
import zipfile
from pathlib import Path

import arpy

from django.utils.html import escape
from django.http import HttpResponseRedirect

from mobsf.MobSF import settings
from mobsf.MobSF.security import (
    sanitize_for_logging,
)
from mobsf.MobSF.security import (
    is_path_traversal,
    is_safe_path,
)
from mobsf.MobSF.utils import (
    EMAIL_REGEX,
    STRINGS_REGEX,
    URL_REGEX,
    append_scan_status,
    is_md5,
    print_n_send_error_response,
    set_permissions,
)
from mobsf.MobSF.views.scanning import (
    add_to_recent_scan,
    handle_uploaded_file,
)
from mobsf.StaticAnalyzer.views.comparer import (
    generic_compare,
)
from mobsf.StaticAnalyzer.views.common.entropy import (
    get_entropies,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)


logger = logging.getLogger(__name__)
RESERVED_FILE_NAMES = [
    'AndroidManifest.xml',
    'resources.arsc',
    'META-INF/MANIFEST.MF',
    'META-INF/CERT.SF',
    'META-INF/CERT.RSA',
    'META-INF/CERT.DSA',
    'classes.dex']
for i in range(2, 50):
    RESERVED_FILE_NAMES.append(f'classes{i}.dex')


def hash_gen(checksum, app_path) -> tuple:
    """Generate and return sha1 and sha256 as a tuple."""
    try:
        msg = 'Generating Hashes'
        logger.info(msg)
        append_scan_status(checksum, msg)
        sha1 = hashlib.sha1()
        sha256 = hashlib.sha256()
        block_size = 65536
        with open(app_path, mode='rb') as afile:
            buf = afile.read(block_size)
            while buf:
                sha1.update(buf)
                sha256.update(buf)
                buf = afile.read(block_size)
        sha1val = sha1.hexdigest()
        sha256val = sha256.hexdigest()
        return sha1val, sha256val
    except Exception as exp:
        msg = 'Failed to generate Hashes'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))


def is_reserved_file_conflict(file_path):
    """Check for reserved file conflict."""
    if any(file_path.startswith(i) and file_path != i for i in RESERVED_FILE_NAMES):
        return True
    return False


def unzip(checksum, app_path, ext_path):
    """Unzip APK.

    Unzip a APK archive while handling encrypted files, reserved file conflicts,
    path traversal (Zip Slip), and permission adjustments. Some of the anti-analysis
    techniques used by malware authors and packers are handled here.

    Args:
        checksum (str): The checksum of the file.
        app_path (str): Path to the ZIP archive.
        ext_path (str): Path to extract the files.

    Returns:
        list: A list of files extracted or an empty list if an error occurs.
    """
    msg = 'Unzipping'
    logger.info(msg)
    append_scan_status(checksum, msg)
    files = []
    original_ext_path = ext_path
    try:
        with zipfile.ZipFile(app_path, 'r') as zipptr:
            files = zipptr.namelist()
            total_size = 0
            stop_fallback_extraction = False
            for fileinfo in zipptr.infolist():
                ext_path = original_ext_path

                # Skip encrypted files
                if fileinfo.flag_bits & 0x1:
                    msg = ('Skipping encrypted file '
                           f'{sanitize_for_logging(fileinfo.filename)}')
                    logger.warning(msg)
                    continue

                file_path = fileinfo.filename.rstrip('/\\')  # Remove trailing slashes

                # Decode the filename
                if not isinstance(file_path, str):
                    file_path = file_path.decode('utf-8', errors='replace')

                # Check for reserved file conflict
                if is_reserved_file_conflict(file_path):
                    ext_path = str(Path(ext_path) / '_conflict_')

                # Handle Zip Slip
                if is_path_traversal(file_path):
                    msg = ('Zip slip detected. skipped extracting'
                           f' {sanitize_for_logging(file_path)}')
                    logger.error(msg)
                    stop_fallback_extraction = True
                    continue

                # Check uncompressed size
                if not fileinfo.is_dir():
                    total_size += fileinfo.file_size
                    if fileinfo.file_size > settings.ZIP_MAX_UNCOMPRESSED_FILE_SIZE:
                        size_mb = fileinfo.file_size / (1024 * 1024)
                        msg = (f'File too large ({size_mb:.2f} MB). Skipping '
                               f'{sanitize_for_logging(file_path)}')
                        logger.warning(msg)
                        continue
                    if total_size > settings.ZIP_MAX_UNCOMPRESSED_TOTAL_SIZE:
                        stop_fallback_extraction = True
                        total_size_mb = total_size / (1024 * 1024)
                        msg = ('Total uncompressed size '
                               f'({total_size_mb:.2f} MB) exceeds limit. '
                               'Aborting extraction.')
                        logger.error(msg)
                        raise Exception(msg)

                # Fix permissions
                if fileinfo.is_dir():
                    # Directories should have rwxr-xr-x (755)
                    # Skip creating directories
                    continue
                else:
                    # Files should have rw-r--r-- (644)
                    fileinfo.external_attr = (0o100644 << 16) | (
                        fileinfo.external_attr & 0xFFFF)

                # Extract the file
                try:
                    zipptr.extract(file_path, ext_path)
                except Exception:
                    logger.warning(
                        'Failed to extract %s', sanitize_for_logging(file_path))
    except Exception as exp:
        msg = f'Unzipping Error - {str(exp)}'
        logger.error(msg)
        append_scan_status(checksum, msg, repr(exp))
        # Do not fallback to OS unzip
        # if the total uncompressed file size is too large
        # or if zip slip is detected
        if stop_fallback_extraction:
            return files
        # Fallback to OS unzip
        ofiles = os_unzip(checksum, app_path, ext_path)
        if not files:
            files = ofiles
    return files


def os_unzip(checksum, app_path, ext_path):
    """Unzip using OS utility."""
    msg = 'Attempting to unzip with OS unzip utility'
    logger.info(msg)
    append_scan_status(checksum, msg)
    try:
        if platform.system() == 'Windows':
            msg = 'Unzipping Error. Not yet implemented in Windows'
            logger.warning(msg)
            append_scan_status(checksum, msg)
            return []
        unzip_b = shutil.which('unzip')
        if not unzip_b:
            msg = 'OS Unzip utility not found'
            logger.warning(msg)
            append_scan_status(checksum, msg)
            return []
        # OS Unzip utility does handle path traversal and absolute paths
        subprocess.call(
            [unzip_b, '-o', '-q', app_path, '-d', ext_path])
        # Set permissions, packed files
        # may not have proper permissions
        set_permissions(ext_path)
        # List files in the unzipped directory
        dat = subprocess.check_output([unzip_b, '-qq', '-l', app_path])
        dat = dat.decode('utf-8').split('\n')
        files_det = ['Length   Date   Time   Name']
        return files_det + dat
    except Exception as exp:
        msg = 'Unzipping Error with OS unzip utility'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return []


def lipo_thin(checksum, src, dst):
    """Thin Fat binary."""
    new_src = None
    try:
        msg = 'Thinning Fat binary'
        logger.info(msg)
        append_scan_status(checksum, msg)
        lipo = shutil.which('lipo')
        out = Path(dst) / (Path(src).stem + '_thin.a')
        new_src = out.as_posix()
        archs = [
            'armv7', 'armv6', 'arm64', 'x86_64',
            'armv4t', 'armv5', 'armv6m', 'armv7f',
            'armv7s', 'armv7k', 'armv7m', 'armv7em',
            'arm64v8']
        for arch in archs:
            args = [
                lipo,
                src,
                '-thin',
                arch,
                '-output',
                new_src]
            out = subprocess.run(
                args,
                stdout=subprocess.DEVNULL,
                stderr=subprocess.STDOUT)
            if out.returncode == 0:
                break
    except Exception as exp:
        msg = 'lipo Fat binary thinning failed'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
    return new_src


def ar_os(src, dst):
    """Extract AR using OS utility."""
    out = b''
    cur = os.getcwd()
    try:
        os.chdir(dst)
        files = subprocess.check_output(
            [shutil.which('ar'), 't', src],
            stderr=subprocess.STDOUT)
        for raw_file in files.decode('utf-8').split('\n'):
            if is_path_traversal(raw_file):
                msg = f'AR slip detected in AR file {src} at {raw_file}'
                logger.error(msg)
                raise ValueError(msg)
        out = subprocess.check_output(
            [shutil.which('ar'), 'x', src],
            stderr=subprocess.STDOUT)
    except ValueError:
        out = b''
    except Exception as exp:
        out = exp.output
    finally:
        os.chdir(cur)
    return out


def ar_extract(checksum, src, dst):
    """Extract AR archive."""
    msg = 'Extracting static library archive'
    logger.info(msg)
    append_scan_status(checksum, msg)
    try:
        ar = arpy.Archive(src)
        ar.read_all_headers()
        for a, val in ar.archived_files.items():
            # Handle archive slip attacks
            filtered = a.decode('utf-8', 'ignore')
            if is_path_traversal(filtered):
                msg = f'Zip slip detected. skipped extracting {filtered}'
                logger.warning(msg)
                append_scan_status(checksum, msg)
                continue
            out = Path(dst) / filtered
            out.write_bytes(val.read())
    except Exception:
        # Possibly dealing with Fat binary, needs Mac host
        msg = 'Failed to extract .a archive'
        logger.warning(msg)
        append_scan_status(checksum, msg)
        # Use os ar utility
        plat = platform.system()
        os_err = 'Possibly a Fat binary. Requires MacOS for Analysis'
        if plat == 'Windows':
            logger.warning(os_err)
            append_scan_status(checksum, os_err)
            return
        msg = 'Using OS ar utility to handle archive'
        logger.info(msg)
        append_scan_status(checksum, msg)
        exp = ar_os(src, dst)
        if len(exp) > 3 and plat == 'Linux':
            # Can't convert FAT binary in Linux
            logger.warning(os_err)
            append_scan_status(checksum, os_err)
            return
        if b'lipo(1)' in exp:
            msg = 'Fat binary archive identified'
            logger.info(msg)
            append_scan_status(checksum, msg)
            # Fat binary archive
            try:
                nw_src = lipo_thin(checksum, src, dst)
                if nw_src:
                    ar_os(nw_src, dst)
            except Exception as exp:
                msg = 'Failed to thin fat archive'
                logger.exception(msg)
                append_scan_status(checksum, msg, repr(exp))


def url_n_email_extract(dat, relative_path):
    """Extract URLs and Emails from Source Code."""
    urls = set()
    emails = set()
    urllist = []
    url_n_file = []
    email_n_file = []
    # URL Extraction
    urllist = URL_REGEX.findall(dat)
    for url in urllist:
        urls.add(url)
    if urls:
        url_n_file.append({
            'urls': list(urls),
            'path': escape(relative_path)})

    # Email Extraction
    for email in EMAIL_REGEX.findall(dat.lower()):
        if not email.startswith('//'):
            emails.add(email)
    if emails:
        email_n_file.append({
            'emails': list(emails),
            'path': escape(relative_path)})
    return urllist, url_n_file, email_n_file


# This is just the first sanity check that triggers generic_compare
@login_required
def compare_apps(request, hash1: str, hash2: str, api=False):
    if hash1 == hash2:
        error_msg = 'Results with same hash cannot be compared'
        return print_n_send_error_response(request, error_msg, api)
    # Second Validation for REST API
    if not (is_md5(hash1) and is_md5(hash2)):
        error_msg = 'Invalid hashes'
        return print_n_send_error_response(request, error_msg, api)
    logger.info(
        'Starting App compare for %s and %s', hash1, hash2)
    return generic_compare(request, hash1, hash2, api)


def get_avg_cvss(findings):
    # Average CVSS Score
    cvss_scores = []
    avg_cvss = 0
    for finding in findings.values():
        find = finding.get('metadata')
        if not find:
            # Hack to support iOS Binary Scan Results
            find = finding
        if find.get('cvss'):
            if find['cvss'] != 0:
                cvss_scores.append(find['cvss'])
    if cvss_scores:
        avg_cvss = round(sum(cvss_scores) / len(cvss_scores), 1)
    if not getattr(settings, 'CVSS_SCORE_ENABLED', False):
        avg_cvss = None
    return avg_cvss


def find_java_source_folder(base_folder: Path):
    # Find the correct java/kotlin source folder for APK/source zip
    # Returns a Tuple of - (SRC_PATH, SRC_TYPE, SRC_SYNTAX)
    return next(p for p in [(base_folder / 'java_source',
                             'java', '*.java'),
                            (base_folder / 'app' / 'src' / 'main' / 'java',
                             'java', '*.java'),
                            (base_folder / 'app' / 'src' / 'main' / 'kotlin',
                             'kotlin', '*.kt'),
                            (base_folder / 'src',
                             'java', '*.java')]
                if p[0].exists())


def is_secret_key(key):
    """Check if the key in the key/value pair is interesting."""
    key_lower = key.lower()
    # Key ends with these strings
    endswith = (
        'api', 'key', 'secret', 'token', 'username',
        'user_name', 'user', 'pass', 'password',
        'private_key', 'access_key',
    )
    # Key contains these strings
    contains = (
        'api_', 'key_', 'aws', 's3_', '_s3', 'secret_',
        'bearer', 'jwt', 'certificate"', 'credential',
        'azure', 'webhook', 'twilio_', 'bitcoin',
        '_auth', 'firebase', 'oauth', 'authorization',
        'private', 'pwd', 'session', 'token_', 'gcp',
    )
    # Key must not contain these strings
    not_string = (
        'label_', 'text', 'hint', 'msg_', 'create_',
        'message', 'new', 'confirm', 'activity_',
        'forgot', 'dashboard_', 'current_', 'signup',
        'sign_in', 'signin', 'title_', 'welcome_',
        'change_', 'this_', 'the_', 'placeholder',
        'invalid_', 'btn_', 'action_', 'prompt_',
        'lable', 'hide_', 'old', 'update', 'error',
        'empty', 'txt_', 'lbl_',
    )
    not_contains_str = any(i in key_lower for i in not_string)
    contains_str = any(i in key_lower for i in contains)
    endswith_str = any(key_lower.endswith(i) for i in endswith)
    return (endswith_str or contains_str) and not not_contains_str


def strings_and_entropies(checksum, src, exts):
    """Get Strings and Entropies."""
    msg = 'Extracting String values and entropies from Code'
    logger.info(msg)
    append_scan_status(checksum, msg)
    data = {
        'strings': set(),
        'secrets': set(),
    }
    try:
        if not (src and src.exists()):
            return data
        excludes = ('\\u0', 'com.google.')
        eslash = ('Ljava', 'Lkotlin', 'kotlin', 'android')
        for p in src.rglob('*'):
            if p.suffix not in exts or not p.exists():
                continue
            matches = STRINGS_REGEX.finditer(
                p.read_text(encoding='utf-8', errors='ignore'),
                re.MULTILINE)
            for match in matches:
                string = match.group()
                if len(string) < 4:
                    continue
                if any(i in string for i in excludes):
                    continue
                if any(i in string and '/' in string for i in eslash):
                    continue
                if not string[0].isalnum():
                    continue
                data['strings'].add(string)
        if data['strings']:
            data['secrets'] = get_entropies(data['strings'])
    except Exception as exp:
        msg = 'Failed to extract String values and entropies from Code'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return data


def get_symbols(symbols):
    all_symbols = []
    for i in symbols:
        for _, val in i.items():
            all_symbols.extend(val)
    return list(set(all_symbols))


@login_required
@permission_required(Permissions.SCAN)
def scan_library(request, checksum):
    """Scan a shared library or framework from path name."""
    try:
        libchecksum = None
        if not is_md5(checksum):
            return print_n_send_error_response(
                request,
                'Invalid MD5')
        relative_path = request.GET['library']
        lib_dir = Path(settings.UPLD_DIR) / checksum

        sfile = lib_dir / relative_path
        if not is_safe_path(lib_dir.as_posix(), sfile.as_posix(), relative_path):
            msg = 'Path Traversal Detected!'
            return print_n_send_error_response(request, msg)
        ext = sfile.suffix
        if not ext and 'Frameworks' in relative_path:
            # Force Dylib on Frameworks
            ext = '.dylib'
        if not sfile.exists():
            msg = 'Library File not found'
            return print_n_send_error_response(request, msg)
        with open(sfile, 'rb') as f:
            libchecksum = handle_uploaded_file(f, ext)
        if ext in [f'.{i}' for i in settings.IOS_EXTS]:
            static_analyzer = 'static_analyzer_ios'
        elif ext == '.appx':
            # Not applicable, but still set it
            static_analyzer = 'windows_static_analyzer'
        elif ext in [f'.{i}' for i in settings.ANDROID_EXTS]:
            static_analyzer = 'static_analyzer'
        else:
            msg = 'Extension not supported'
            return print_n_send_error_response(request, msg)
        data = {
            'analyzer': static_analyzer,
            'status': 'success',
            'hash': libchecksum,
            'scan_type': ext.replace('.', ''),
            'file_name': sfile.name,
        }
        add_to_recent_scan(data)
        return HttpResponseRedirect(f'/{static_analyzer}/{libchecksum}/')
    except Exception:
        msg = 'Failed to perform Static Analysis of library'
        logger.exception(msg)
        return print_n_send_error_response(request, msg)



"""Handle JAR and AAR files."""
import logging
from pathlib import Path

from django.conf import settings
from django.shortcuts import render

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal
from mobsf.MalwareAnalyzer.views.android import (
    permissions,
)
from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_avg_cvss,
    hash_gen,
    unzip,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.StaticAnalyzer.views.common.appsec import (
    get_android_dashboard,
)
from mobsf.StaticAnalyzer.views.android.manifest_analysis import (
    manifest_analysis,
)
from mobsf.StaticAnalyzer.views.android.manifest_utils import (
    extract_manifest_data,
    get_parsed_manifest,
)
from mobsf.StaticAnalyzer.views.android.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.android.cert_analysis import (
    cert_info,
    get_hardcoded_cert_keystore,
)
from mobsf.StaticAnalyzer.models import (
    StaticAnalyzerAndroid,
)
from mobsf.StaticAnalyzer.views.android.code_analysis import code_analysis
from mobsf.StaticAnalyzer.views.android.converter import (
    apk_2_java,
)
from mobsf.StaticAnalyzer.views.android.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import MalwareDomainCheck
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

APK_TYPE = 'apk'
logger = logging.getLogger(__name__)


def common_analysis(request, app_dic, rescan, api, analysis_type):
    checksum = app_dic['md5']
    app_dic['app_file'] = f'{app_dic["md5"]}.{analysis_type}'  # NEW FILENAME
    app_dic['app_path'] = (app_dic['app_dir'] / app_dic['app_file']).as_posix()
    app_dic['app_dir'] = app_dic['app_dir'].as_posix() + '/'
    db_entry = StaticAnalyzerAndroid.objects.filter(MD5=checksum)
    if db_entry.exists() and not rescan:
        context = get_context_from_db_entry(db_entry)
    else:
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(
                request,
                'Permission Denied',
                False)
        append_scan_status(checksum, 'init')
        # Analysis Starts here
        app_dic['size'] = f'{str(file_size(app_dic["app_path"]))}MB'
        app_dic['sha1'], app_dic['sha256'] = hash_gen(
            checksum,
            app_dic['app_path'])
        app_dic['zipped'] = analysis_type
        app_dic['files'] = unzip(
            checksum,
            app_dic['app_path'],
            app_dic['app_dir'])
        logger.info('%s Extracted', analysis_type.upper())
        if not app_dic['files']:
            return print_n_send_error_response(
                request,
                f'{analysis_type.upper()} file is invalid or corrupt',
                api)
        get_hardcoded_cert_keystore(app_dic)
        app_dic['playstore'] = {'error': True}
        if analysis_type == 'aar':
            # AAR has manifest and sometimes certificate
            get_parsed_manifest(app_dic)
            man_data_dic = extract_manifest_data(app_dic)
            man_an_dic = manifest_analysis(app_dic, man_data_dic)
            # Malware Permission check
            mal_perms = permissions.check_malware_permission(
                checksum,
                man_data_dic['perm'])
            man_an_dic['malware_permissions'] = mal_perms

            cert_dic = cert_info(app_dic, man_data_dic)
        else:
            app_dic['manifest_file'] = None
            app_dic['manifest_parsed_xml'] = None
            app_dic['manifest_namespace'] = None
            man_data_dic = {
                'services': [],
                'activities': [],
                'receivers': [],
                'providers': [],
                'libraries': [],
                'categories': [],
                'perm': {},
                'packagename': app_dic['app_name'],
                'mainactivity': '',
                'min_sdk': '',
                'max_sdk': '',
                'target_sdk': '',
                'androver': '',
                'androvername': '',
                'icons': [],
            }
            man_an_dic = {
                'manifest_anal': [],
                'exported_act': [],
                'exported_cnt': {
                    'exported_activities': 0,
                    'exported_services': 0,
                    'exported_receivers': 0,
                    'exported_providers': 0,
                },
                'browsable_activities': {},
                'permissions': {},
                'network_security': {
                    'network_findings': [],
                    'network_summary': {},
                },
                'malware_permissions': {},
            }
            cert_dic = {
                'certificate_info': '',
                'certificate_findings': [],
                'certificate_summary': {},
            }
        app_dic['real_name'] = ''
        elf_dict = library_analysis(
            checksum,
            app_dic['app_dir'],
            'elf')
        tracker = Trackers.Trackers(
            checksum,
            app_dic['app_dir'],
            app_dic['tools_dir'])
        tracker_res = tracker.get_trackers()
        apk_2_java(
            checksum,
            app_dic['app_path'],
            app_dic['app_dir'],
            settings.DOWNLOADED_TOOLS_DIR)
        code_an_dic = code_analysis(
            checksum,
            app_dic['app_dir'],
            APK_TYPE,
            app_dic['manifest_file'],
            man_data_dic['perm'])
        obfuscated_check(
            checksum,
            app_dic['app_dir'],
            code_an_dic)
        # Get the strings and metadata
        get_strings_metadata(
            app_dic,
            elf_dict['elf_strings'],
            ['.java'],
            code_an_dic)
        # Firebase DB Check
        code_an_dic['firebase'] = firebase_analysis(
            checksum,
            code_an_dic)
        # Domain Extraction and Malware Check
        code_an_dic['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_an_dic['urls_list'])

        context = save_get_ctx(
            app_dic,
            man_data_dic,
            man_an_dic,
            code_an_dic,
            cert_dic,
            elf_dict['elf_analysis'],
            {},
            tracker_res,
            rescan,
        )
    context['appsec'] = get_android_dashboard(context, True)
    context['average_cvss'] = get_avg_cvss(
        context['code_analysis'])
    context['dynamic_analysis_done'] = False
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(
            app_dic['app_path'])
    template = 'static_analysis/android_binary_analysis.html'
    if api:
        return context
    else:
        return render(request, template, context)


def jar_analysis(request, app_dic, rescan, api):
    return common_analysis(request, app_dic, rescan, api, 'jar')


def aar_analysis(request, app_dic, rescan, api):
    return common_analysis(request, app_dic, rescan, api, 'aar')


def obfuscated_check(checksum, src, code_an_dic):
    """Check if JAR/AAR is obfuscated."""
    msg = 'Checking for Obfuscation'
    logger.info(msg)
    append_scan_status(checksum, msg)
    metadata = {
        'cvss': 0,
        'cwe': '',
        'owasp-mobile': '',
        'masvs': '',
        'ref': '',
        'description': (
            'The binary might not be obfuscated.'
            ' LocalVariableTable is present in class file.'),
        'severity': 'info',
    }
    try:
        app_dir = Path(src)
        # Extract all jar files
        for j in app_dir.rglob('*.jar'):
            if not j.is_file():
                continue
            out = app_dir / f'{j.name}_out'
            if not out.exists():
                unzip(checksum, j, out)
        # Search all class files
        for i in app_dir.rglob('*.class'):
            if not i.is_file():
                continue
            cls_dat = i.read_text(
                encoding='utf-8', errors='ignore')
            if 'LocalVariableTable' in cls_dat:
                code_an_dic['findings']['aar_class_obfuscation'] = {
                    'files': {i.name: '1,1'},
                    'metadata': metadata,
                }
                return
    except Exception as exp:
        msg = 'Obfuscation Check Failed'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    metadata['description'] = (
        'The binary might be obfuscated.'
        ' LocalVariableTable is absent in class file.')
    code_an_dic['findings']['aar_class_obfuscation'] = {
        'files': {},
        'metadata': metadata,
    }
    return



from django.urls import re_path

from mobsf.DynamicAnalyzer.views.common import device
from mobsf.DynamicAnalyzer.views.common.frida import views as frida
from mobsf.DynamicAnalyzer.views.android import dynamic_analyzer as dz
from mobsf.DynamicAnalyzer.views.android import (
    operations,
    report,
    tests_common,
    tests_frida,
)
from mobsf.DynamicAnalyzer.views.ios import dynamic_analyzer as idz
from mobsf.DynamicAnalyzer.views.ios import (
    corellium_instance as instance,
    report as ios_view_report,
    tests_frida as ios_tests_frida,
)
from mobsf.DynamicAnalyzer.views.ios.device import (
    dynamic_analyzer as ios_device,
)
from mobsf.DynamicAnalyzer.views.ios.device import (
    report as device_report,
)
from mobsf.MobSF import utils
from mobsf.MobSF.security import (
    init_exec_hooks,
    store_exec_hashes_at_first_run,
)
from mobsf.MobSF.views import (
    authentication,
    authorization,
    home,
    saml2,
)
from mobsf.MobSF.views.api import api_static_analysis as api_sz
from mobsf.MobSF.views.api import api_android_dynamic_analysis as api_dz
from mobsf.MobSF.views.api import api_ios_dynamic_analysis as api_idz
from mobsf.MobSF.views.api import api_ios_device_analysis as api_iddz
from mobsf.StaticAnalyzer import tests
from mobsf.StaticAnalyzer.views.common import (
    appsec,
    async_task,
    pdf,
    shared_func,
    suppression,
)
from mobsf.StaticAnalyzer.views.android.views import (
    find,
    manifest_view,
    source_tree,
    view_source,
)
from mobsf.StaticAnalyzer.views.windows import windows
from mobsf.StaticAnalyzer.views.android import static_analyzer as android_sa
from mobsf.StaticAnalyzer.views.ios import static_analyzer as ios_sa
from mobsf.StaticAnalyzer.views.ios.views import view_source as io_view_source

from . import settings

bundle_id_regex = r'(?P<bundle_id>.+)$'
checksum_regex = r'(?P<checksum>[0-9a-f]{32})'
paginate = r'(?P<page_size>[0-9]{1,10})/(?P<page_number>[0-9]{1,10})'

urlpatterns = [
    re_path(r'^login/$',
            authentication.login_view,
            name='login'),
    re_path(r'^logout$',
            authentication.logout_view,
            name='logout'),
    re_path(r'^change_password/$',
            authentication.change_password,
            name='change_password'),
    re_path(r'^users/$',
            authorization.users,
            name='users'),
    re_path(r'^create_user/$',
            authorization.create_user,
            name='create_user'),
    re_path(r'^delete_user/$',
            authorization.delete_user,
            name='delete_user'),
    # SAML2
    re_path(r'^sso/$',
            saml2.saml_login,
            name='saml_login'),
    re_path(r'^sso/acs/$',
            saml2.saml_acs,
            name='saml_acs'),
    # REST API
    # Static Analysis
    re_path(r'^api/v1/upload$', api_sz.api_upload),
    re_path(r'^api/v1/scan$', api_sz.api_scan),
    re_path(r'^api/v1/search$', api_sz.api_search),
    re_path(r'^api/v1/scan_logs$', api_sz.api_scan_logs),
    re_path(r'^api/v1/tasks$', api_sz.api_tasks),
    re_path(r'^api/v1/delete_scan$', api_sz.api_delete_scan),
    re_path(r'^api/v1/download_pdf$', api_sz.api_pdf_report),
    re_path(r'^api/v1/report_json$', api_sz.api_json_report),
    re_path(r'^api/v1/view_source$', api_sz.api_view_source,
            name='api_view_source'),
    re_path(r'^api/v1/scans$', api_sz.api_recent_scans),
    re_path(r'^api/v1/compare$', api_sz.api_compare),
    re_path(r'^api/v1/scorecard$', api_sz.api_scorecard),
    # Static Suppression
    re_path(r'^api/v1/suppress_by_rule$', api_sz.api_suppress_by_rule_id),
    re_path(r'^api/v1/suppress_by_files$', api_sz.api_suppress_by_files),
    re_path(r'^api/v1/list_suppressions$', api_sz.api_list_suppressions),
    re_path(r'^api/v1/delete_suppression$', api_sz.api_delete_suppression),
    # Dynamic Analysis
    re_path(r'^api/v1/dynamic/get_apps$', api_dz.api_get_apps),
    re_path(r'^api/v1/dynamic/start_analysis$', api_dz.api_start_analysis),
    re_path(r'^api/v1/dynamic/stop_analysis$', api_dz.api_stop_analysis),
    re_path(r'^api/v1/dynamic/report_json$', api_dz.api_dynamic_report),
    # Android Specific
    re_path(r'^api/v1/android/logcat$', api_dz.api_logcat),
    re_path(r'^api/v1/android/mobsfy$', api_dz.api_mobsfy),
    re_path(r'^api/v1/android/adb_command$', api_dz.api_adb_execute),
    re_path(r'^api/v1/android/root_ca$', api_dz.api_root_ca),
    re_path(r'^api/v1/android/global_proxy$', api_dz.api_global_proxy),
    re_path(r'^api/v1/android/activity$', api_dz.api_act_tester),
    re_path(r'^api/v1/android/start_activity$', api_dz.api_start_activity),
    re_path(r'^api/v1/android/tls_tests$', api_dz.api_tls_tester),
    # Frida
    re_path(r'^api/v1/frida/instrument$', api_dz.api_instrument),
    re_path(r'^api/v1/frida/api_monitor$', api_dz.api_api_monitor),
    re_path(r'^api/v1/frida/get_dependencies$', api_dz.api_get_dependencies),
    # Shared
    re_path(r'^api/v1/frida/logs$', api_dz.api_frida_logs),
    re_path(r'^api/v1/frida/list_scripts$', api_dz.api_list_frida_scripts),
    re_path(r'^api/v1/frida/get_script$', api_dz.api_get_script_content),
    re_path(r'^api/v1/dynamic/view_source$', api_dz.api_dynamic_view_file),
    # iOS Corellium
    re_path(r'^api/v1/ios/corellium_supported_models$',
            api_idz.api_corellium_get_supported_models),
    re_path(r'^api/v1/ios/corellium_ios_versions$',
            api_idz.api_corellium_get_supported_ios_versions),
    re_path(r'^api/v1/ios/corellium_create_ios_instance$',
            api_idz.api_corellium_create_ios_instance),
    re_path(r'^api/v1/ios/dynamic_analysis$',
            api_idz.api_ios_dynamic_analysis),
    re_path(r'^api/v1/ios/corellium_start_instance$',
            api_idz.api_corellium_start_instance),
    re_path(r'^api/v1/ios/corellium_stop_instance$',
            api_idz.api_corellium_stop_instance),
    re_path(r'^api/v1/ios/corellium_unpause_instance$',
            api_idz.api_corellium_unpause_instance),
    re_path(r'^api/v1/ios/corellium_reboot_instance$',
            api_idz.api_corellium_reboot_instance),
    re_path(r'^api/v1/ios/corellium_destroy_instance$',
            api_idz.api_corellium_destroy_instance),
    re_path(r'^api/v1/ios/corellium_list_apps$',
            api_idz.api_corellium_instance_list_apps),
    re_path(r'^api/v1/ios/setup_environment$',
            api_idz.api_setup_environment),
    re_path(r'^api/v1/ios/dynamic_analyzer$',
            api_idz.api_ios_dynamic_analyzer),
    re_path(r'^api/v1/ios/run_app$',
            api_idz.api_run_app),
    re_path(r'^api/v1/ios/stop_app$',
            api_idz.api_stop_app),
    re_path(r'^api/v1/ios/remove_app$',
            api_idz.api_remove_app),
    re_path(r'^api/v1/ios/take_screenshot$',
            api_idz.api_take_screenshot),
    re_path(r'^api/v1/ios/get_app_container_path$',
            api_idz.api_get_app_container_path),
    re_path(r'^api/v1/ios/network_capture$',
            api_idz.api_network_capture),
    re_path(r'^api/v1/ios/live_pcap_download$',
            api_idz.api_live_pcap_download),
    re_path(r'^api/v1/ios/ssh_execute$',
            api_idz.api_ssh_execute),
    re_path(r'^api/v1/ios/download_app_data$',
            api_idz.api_download_app_data),
    re_path(r'^api/v1/ios/instance_input$',
            api_idz.api_instance_input),
    re_path(r'^api/v1/ios/system_logs$',
            api_idz.api_system_logs),
    re_path(r'^api/v1/ios/file_upload$',
            api_idz.api_device_file_upload),
    re_path(r'^api/v1/ios/file_download$',
            api_idz.api_device_file_download),
    # Frida
    re_path(r'^api/v1/frida/ios_instrument$', api_idz.api_ios_instrument),
    re_path(r'^api/v1/dynamic/ios_report_json$', api_idz.api_ios_view_report),
    # iOS Jailbroken Device
    re_path(r'^api/v1/ios/device_dynamic_analysis$',
            api_iddz.api_device_dynamic_analysis),
    re_path(r'^api/v1/ios/device_get$',
            api_iddz.api_device_get),
    re_path(r'^api/v1/ios/device_install_ipa$',
            api_iddz.api_device_install_ipa),
    re_path(r'^api/v1/ios/device_dynamic_analyzer$',
            api_iddz.api_device_dynamic_analyzer),
    re_path(r'^api/v1/ios/device_file_upload$',
            api_iddz.api_device_file_upload),
    re_path(r'^api/v1/ios/device_ssh_execute$',
            api_iddz.api_device_ssh_execute),
    re_path(r'^api/v1/ios/device_instrument$',
            api_iddz.api_device_instrument),
    re_path(r'^api/v1/ios/device_system_logs$',
            api_iddz.api_device_system_logs),
    re_path(r'^api/v1/ios/device_ps$',
            api_iddz.api_device_ps),
    re_path(r'^api/v1/ios/device_file_download$',
            api_iddz.api_device_file_download),
    re_path(r'^api/v1/ios/device_api_monitor$',
            api_iddz.api_device_api_monitor),
    re_path(r'^api/v1/ios/device_download_app_data$',
            api_iddz.api_device_download_app_data),
    re_path(r'^api/v1/ios/device_report_json$',
            api_iddz.api_device_report_json),
]
if settings.API_ONLY == '0':
    urlpatterns.extend([
        # General
        re_path(r'^$', home.index, name='home'),
        re_path(r'^upload/$', home.Upload.as_view, name='upload'),
        re_path(r'^download/', home.download, name='download'),
        re_path(fr'^download_binary/{checksum_regex}/$',
                home.download_binary,
                name='download_binary'),
        re_path(r'^download_scan/', home.download_apk, name='download_scan'),
        re_path(r'^generate_downloads/$',
                home.generate_download,
                name='generate_downloads'),
        re_path(r'^about$', home.about, name='about'),
        re_path(r'^donate$', home.donate, name='donate'),
        re_path(r'^api_docs$', home.api_docs, name='api_docs'),
        re_path(r'^recent_scans/$', home.recent_scans, name='recent'),
        re_path(fr'^recent_scans/{paginate}/$',
                home.recent_scans,
                name='scans_paginated'),
        re_path(r'^delete_scan/$', home.delete_scan, name='delete_scan'),
        re_path(r'^search$', home.search),
        re_path(r'^status/$', home.scan_status, name='status'),
        re_path(r'^error/$', home.error, name='error'),
        re_path(r'^zip_format/$', home.zip_format),
        re_path(r'^robots.txt$', home.robots_txt),
        re_path(r'^dynamic_analysis/$', home.dynamic_analysis, name='dynamic'),
        re_path(r'^tasks$',
                async_task.list_tasks,
                name='list_tasks'),
        # Static Analysis
        # Android
        re_path(fr'^static_analyzer/{checksum_regex}/$',
                android_sa.static_analyzer,
                name='static_analyzer'),
        # Remove this is version 4/5
        re_path(r'^source_code/$', source_tree.run, name='tree_view'),
        re_path(r'^view_file/$', view_source.run, name='view_source'),
        re_path(r'^find/$', find.run, name='find_files'),
        re_path(fr'^manifest_view/{checksum_regex}/$',
                manifest_view.run,
                name='manifest_view'),
        # IOS
        re_path(fr'^static_analyzer_ios/{checksum_regex}/$',
                ios_sa.static_analyzer_ios,
                name='static_analyzer_ios'),
        re_path(r'^view_file_ios/$',
                io_view_source.run,
                name='view_file_ios'),
        # Windows
        re_path(fr'^static_analyzer_windows/{checksum_regex}/$',
                windows.staticanalyzer_windows,
                name='static_analyzer_windows'),
        # Shared
        re_path(fr'^pdf/{checksum_regex}/$', pdf.pdf, name='pdf'),
        re_path(fr'^appsec_dashboard/{checksum_regex}/$',
                appsec.appsec_dashboard,
                name='appsec_dashboard'),
        # Suppression
        re_path(r'^suppress_by_rule/$',
                suppression.suppress_by_rule_id,
                name='suppress_by_rule'),
        re_path(r'^suppress_by_files/$',
                suppression.suppress_by_files,
                name='suppress_by_files'),
        re_path(r'^list_suppressions/$',
                suppression.list_suppressions,
                name='list_suppressions'),
        re_path(r'^delete_suppression/$',
                suppression.delete_suppression,
                name='delete_suppression'),
        # App Compare
        re_path(r'^compare/(?P<hash1>[0-9a-f]{32})/(?P<hash2>[0-9a-f]{32})/$',
                shared_func.compare_apps),
        # Relative Shared & Dynamic Library scan
        re_path(fr'^scan_library/{checksum_regex}$',
                shared_func.scan_library,
                name='scan_library'),
        # Dynamic Analysis
        re_path(r'^android/dynamic_analysis/$',
                dz.android_dynamic_analysis,
                name='dynamic_android'),
        re_path(fr'^android_dynamic/{checksum_regex}$',
                dz.dynamic_analyzer,
                name='dynamic_analyzer'),
        re_path(r'^httptools$',
                dz.httptools_start,
                name='httptools'),
        re_path(r'^logcat/$', dz.logcat),
        re_path(fr'^static_scan/{checksum_regex}$',
                dz.trigger_static_analysis,
                name='static_scan'),
        # Android Operations
        re_path(r'^run_apk/$', operations.run_apk),
        re_path(r'^mobsfy/$', operations.mobsfy, name='mobsfy'),
        re_path(r'^screenshot/$', operations.take_screenshot),
        re_path(r'^execute_adb/$', operations.execute_adb),
        re_path(r'^screen_cast/$',
                operations.screen_cast,
                name='screen_cast'),
        re_path(r'^touch_events/$',
                operations.touch,
                name='android_touch'),
        re_path(r'^get_component/$', operations.get_component),
        re_path(r'^mobsf_ca/$', operations.mobsf_ca),
        re_path(r'^global_proxy/$', operations.global_proxy),
        # Dynamic Tests
        re_path(r'^activity_tester/$', tests_common.activity_tester),
        re_path(r'^start_activity/$',
                tests_common.start_activity,
                name='start_activity'),
        re_path(r'^start_deeplink/$',
                tests_common.start_deeplink,
                name='start_deeplink'),
        re_path(r'^download_data/$', tests_common.download_data),
        re_path(r'^collect_logs/$', tests_common.collect_logs),
        re_path(r'^tls_tests/$', tests_common.tls_tests),
        # Frida
        re_path(r'^frida_instrument/$',
                tests_frida.instrument,
                name='android_instrument'),
        re_path(r'^live_api/$',
                tests_frida.live_api,
                name='live_api'),
        re_path(r'^frida_logs/$',
                frida.frida_logs,
                name='frida_logs'),
        re_path(r'^get_dependencies/$', tests_frida.get_runtime_dependencies),
        # Report
        re_path(fr'^dynamic_report/{checksum_regex}$',
                report.view_report,
                name='dynamic_report'),
        # Shared
        re_path(r'^list_frida_scripts/$',
                frida.list_frida_scripts,
                name='list_frida_scripts'),
        re_path(r'^get_script_content/$',
                frida.get_script_content,
                name='get_script'),
        re_path(r'^dynamic_view_file/$',
                device.view_file,
                name='dynamic_view_file'),
        # iOS Dynamic Analysis
        re_path(r'^ios/dynamic_analysis/$',
                idz.dynamic_analysis,
                name='dynamic_ios'),
        re_path(r'^ios/create_vm_instance/$',
                instance.create_vm_instance,
                name='create_vm_instance'),
        re_path(r'^ios/get_supported_models/$',
                instance.get_supported_models,
                name='get_supported_models'),
        re_path(r'^ios/get_supported_os/$',
                instance.get_supported_os,
                name='get_supported_os'),
        re_path(r'^ios/start_instance/$',
                instance.start_instance,
                name='start_instance'),
        re_path(r'^ios/stop_instance/$',
                instance.stop_instance,
                name='stop_instance'),
        re_path(r'^ios/unpause_instance/$',
                instance.unpause_instance,
                name='unpause_instance'),
        re_path(r'^ios/reboot_instance/$',
                instance.reboot_instance,
                name='reboot_instance'),
        re_path(r'^ios/destroy_instance/$',
                instance.destroy_instance,
                name='destroy_instance'),
        re_path(r'^ios/list_apps/$',
                instance.list_apps,
                name='list_apps'),
        re_path(fr'^ios/setup_environment/{checksum_regex}$',
                instance.setup_environment,
                name='setup_environment'),
        re_path(r'^ios/dynamic_analyzer/$',
                idz.dynamic_analyzer,
                name='dynamic_analyzer_ios'),
        re_path(r'^ios/run_app/$',
                instance.run_app,
                name='run_app'),
        re_path(r'^ios/remove_app/$',
                instance.remove_app,
                name='remove_app'),
        re_path(r'^ios/take_screenshot/$',
                instance.take_screenshot,
                name='take_screenshot'),
        re_path(r'^ios/network_capture/$',
                instance.network_capture,
                name='network_capture'),
        re_path(r'^ios/live_pcap_download/$',
                instance.live_pcap_download,
                name='ios_live_pcap_download'),
        re_path(r'^ios/ssh_execute/$',
                instance.ssh_execute,
                name='ssh_execute'),
        re_path(r'^ios/upload_file/$',
                instance.upload_file,
                name='upload_file'),
        re_path(r'^ios/download_file/$',
                instance.download_file,
                name='download_file'),
        re_path(r'^ios/touch/$',
                instance.touch,
                name='ios_touch'),
        re_path(r'^ios/get_container_path/$',
                instance.get_container_path,
                name='get_container_path'),
        re_path(r'^ios/system_logs/$',
                instance.system_logs,
                name='ios_system_logs'),
        re_path(fr'^ios/download_data/{bundle_id_regex}',
                instance.download_data,
                name='ios_download_data'),
        re_path(r'^ios/instrument/$',
                ios_tests_frida.ios_instrument,
                name='ios_instrument'),
        re_path(fr'^ios/view_report/{bundle_id_regex}',
                ios_view_report.ios_view_report,
                name='ios_view_report'),
        # iOS Device
        re_path(r'^ios/dynamic_analysis_device/$',
                ios_device.dynamic_analysis_device,
                name='dynamic_ios_device'),
        re_path(r'^ios/ios_device/$',
                ios_device.get_ios_device,
                name='get_ios_device'),
        re_path(r'^ios/install_ipa_device/$',
                ios_device.install_ipa_device,
                name='install_ipa_device'),
        re_path(r'^ios/dynamic_analyzer_device/$',
                ios_device.dynamic_analyzer_device,
                name='dynamic_analyzer_device'),
        re_path(r'^ios/upload_file_device/$',
                ios_device.upload_file_device,
                name='upload_file_device'),
        re_path(r'^ios/ssh_execute_device/$',
                ios_device.ssh_execute_device,
                name='ssh_execute_device'),
        re_path(r'^ios/instrument_device/$',
                ios_device.ios_instrument_device,
                name='ios_instrument_device'),
        re_path(r'^ios/system_logs_device/$',
                ios_device.system_logs_device,
                name='system_logs_device'),
        re_path(r'^ios/ps_device/$',
                ios_device.ps_device,
                name='ps_device'),
        re_path(r'^ios/download_file_device/$',
                device_report.download_file_device,
                name='download_file_device'),
        re_path(r'^ios/api_monitor/$',
                device_report.ios_api_monitor,
                name='ios_api_monitor'),
        re_path(fr'^ios/download_data_device/{bundle_id_regex}',
                device_report.download_data_device,
                name='download_data_device'),
        re_path(fr'^ios/view_report_device/{bundle_id_regex}',
                device_report.view_report_device,
                name='view_report_device'),
        # Test
        re_path(r'^tests/$', tests.start_test),
    ])

utils.print_version()
init_exec_hooks()
store_exec_hashes_at_first_run()



# -*- coding: utf_8 -*-
import logging

from django.conf import settings
from django.db.models import QuerySet

from mobsf.MobSF.utils import (
    append_scan_status,
    get_scan_logs,
    python_dict,
    python_list,
)
from mobsf.MobSF.views.home import update_scan_timestamp
from mobsf.StaticAnalyzer.models import StaticAnalyzerAndroid
from mobsf.StaticAnalyzer.models import RecentScansDB
from mobsf.StaticAnalyzer.views.common.suppression import (
    process_suppression,
    process_suppression_manifest,
)

"""Module holding the functions for the db."""


logger = logging.getLogger(__name__)


def get_context_from_db_entry(db_entry: QuerySet) -> dict:
    """Return the context for APK/ZIP from DB."""
    try:
        msg = 'Analysis is already Done. Fetching data from the DB...'
        logger.info(msg)
        package = db_entry[0].PACKAGE_NAME
        code = process_suppression(
            python_dict(db_entry[0].CODE_ANALYSIS),
            package)
        manifest_analysis = process_suppression_manifest(
            python_list(db_entry[0].MANIFEST_ANALYSIS),
            package)
        context = {
            'version': settings.MOBSF_VER,
            'title': 'Static Analysis',
            'file_name': db_entry[0].FILE_NAME,
            'app_name': db_entry[0].APP_NAME,
            'app_type': db_entry[0].APP_TYPE,
            'size': db_entry[0].SIZE,
            'md5': db_entry[0].MD5,
            'sha1': db_entry[0].SHA1,
            'sha256': db_entry[0].SHA256,
            'package_name': package,
            'main_activity': db_entry[0].MAIN_ACTIVITY,
            'exported_activities': db_entry[0].EXPORTED_ACTIVITIES,
            'browsable_activities': python_dict(
                db_entry[0].BROWSABLE_ACTIVITIES),
            'activities': python_list(db_entry[0].ACTIVITIES),
            'receivers': python_list(db_entry[0].RECEIVERS),
            'providers': python_list(db_entry[0].PROVIDERS),
            'services': python_list(db_entry[0].SERVICES),
            'libraries': python_list(db_entry[0].LIBRARIES),
            'target_sdk': db_entry[0].TARGET_SDK,
            'max_sdk': db_entry[0].MAX_SDK,
            'min_sdk': db_entry[0].MIN_SDK,
            'version_name': db_entry[0].VERSION_NAME,
            'version_code': db_entry[0].VERSION_CODE,
            'icon_path': db_entry[0].ICON_PATH,
            'permissions': python_dict(db_entry[0].PERMISSIONS),
            'malware_permissions': python_dict(
                db_entry[0].MALWARE_PERMISSIONS),
            'certificate_analysis': python_dict(
                db_entry[0].CERTIFICATE_ANALYSIS),
            'manifest_analysis': manifest_analysis,
            'network_security': python_dict(db_entry[0].NETWORK_SECURITY),
            'binary_analysis': python_list(db_entry[0].BINARY_ANALYSIS),
            'file_analysis': python_list(db_entry[0].FILE_ANALYSIS),
            'android_api': python_dict(db_entry[0].ANDROID_API),
            'code_analysis': code,
            'niap_analysis': python_dict(db_entry[0].NIAP_ANALYSIS),
            'permission_mapping': python_dict(db_entry[0].PERMISSION_MAPPING),
            'urls': python_list(db_entry[0].URLS),
            'domains': python_dict(db_entry[0].DOMAINS),
            'emails': python_list(db_entry[0].EMAILS),
            'strings': python_dict(db_entry[0].STRINGS),
            'firebase_urls': python_list(db_entry[0].FIREBASE_URLS),
            'files': python_list(db_entry[0].FILES),
            'exported_count': python_dict(db_entry[0].EXPORTED_COUNT),
            'apkid': python_dict(db_entry[0].APKID),
            'behaviour': python_dict(db_entry[0].QUARK),
            'trackers': python_dict(db_entry[0].TRACKERS),
            'playstore_details': python_dict(db_entry[0].PLAYSTORE_DETAILS),
            'secrets': python_list(db_entry[0].SECRETS),
            'logs': get_scan_logs(db_entry[0].MD5),
            'sbom': python_dict(db_entry[0].SBOM),
        }
        return context
    except Exception:
        msg = 'Fetching data from the DB failed.'
        logger.exception(msg)
        return None


def get_context_from_analysis(app_dic,
                              man_data_dic,
                              man_an_dic,
                              code_an_dic,
                              cert_dic,
                              bin_anal,
                              apk_id,
                              trackers) -> dict:
    """Get the context for APK/ZIP from analysis results."""
    try:
        package = man_data_dic['packagename']
        code = process_suppression(
            code_an_dic['findings'],
            package)
        manifest_analysis = process_suppression_manifest(
            man_an_dic['manifest_anal'],
            package)
        context = {
            'title': 'Static Analysis',
            'version': settings.MOBSF_VER,
            'file_name': app_dic['app_name'],
            'app_name': app_dic['real_name'],
            'app_type': app_dic['zipped'],
            'size': app_dic['size'],
            'md5': app_dic['md5'],
            'sha1': app_dic['sha1'],
            'sha256': app_dic['sha256'],
            'package_name': package,
            'main_activity': man_data_dic['mainactivity'],
            'exported_activities': man_an_dic['exported_act'],
            'browsable_activities': man_an_dic['browsable_activities'],
            'activities': man_data_dic['activities'],
            'receivers': man_data_dic['receivers'],
            'providers': man_data_dic['providers'],
            'services': man_data_dic['services'],
            'libraries': man_data_dic['libraries'],
            'target_sdk': man_data_dic['target_sdk'],
            'max_sdk': man_data_dic['max_sdk'],
            'min_sdk': man_data_dic['min_sdk'],
            'version_name': man_data_dic['androvername'],
            'version_code': man_data_dic['androver'],
            'icon_path': app_dic['icon_path'],
            'certificate_analysis': cert_dic,
            'permissions': man_an_dic['permissions'],
            'malware_permissions': man_an_dic['malware_permissions'],
            'manifest_analysis': manifest_analysis,
            'network_security': man_an_dic['network_security'],
            'binary_analysis': bin_anal,
            'file_analysis': app_dic['file_analysis'],
            'android_api': code_an_dic['api'],
            'code_analysis': code,
            'niap_analysis': code_an_dic['niap'],
            'permission_mapping': code_an_dic['perm_mappings'],
            'urls': code_an_dic['urls'],
            'domains': code_an_dic['domains'],
            'emails': code_an_dic['emails'],
            'strings': code_an_dic['strings'],
            'firebase_urls': code_an_dic['firebase'],
            'files': app_dic['files'],
            'exported_count': man_an_dic['exported_cnt'],
            'apkid': apk_id,
            'behaviour': code_an_dic['behaviour'],
            'trackers': trackers,
            'playstore_details': app_dic['playstore'],
            'secrets': code_an_dic['secrets'],
            'logs': get_scan_logs(app_dic['md5']),
            'sbom': code_an_dic['sbom'],
        }
        return context
    except Exception as exp:
        msg = 'Rendering to Template failed.'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))


def save_or_update(update_type,
                   app_dic,
                   man_data_dic,
                   man_an_dic,
                   code_an_dic,
                   cert_dic,
                   bin_anal,
                   apk_id,
                   trackers) -> None:
    """Save/Update an APK/ZIP DB entry."""
    try:
        values = {
            'FILE_NAME': app_dic['app_name'],
            'APP_NAME': app_dic['real_name'],
            'APP_TYPE': app_dic['zipped'],
            'SIZE': app_dic['size'],
            'MD5': app_dic['md5'],
            'SHA1': app_dic['sha1'],
            'SHA256': app_dic['sha256'],
            'PACKAGE_NAME': man_data_dic['packagename'],
            'MAIN_ACTIVITY': man_data_dic['mainactivity'],
            'EXPORTED_ACTIVITIES': man_an_dic['exported_act'],
            'BROWSABLE_ACTIVITIES': man_an_dic['browsable_activities'],
            'ACTIVITIES': man_data_dic['activities'],
            'RECEIVERS': man_data_dic['receivers'],
            'PROVIDERS': man_data_dic['providers'],
            'SERVICES': man_data_dic['services'],
            'LIBRARIES': man_data_dic['libraries'],
            'TARGET_SDK': man_data_dic['target_sdk'],
            'MAX_SDK': man_data_dic['max_sdk'],
            'MIN_SDK': man_data_dic['min_sdk'],
            'VERSION_NAME': man_data_dic['androvername'],
            'VERSION_CODE': man_data_dic['androver'],
            'ICON_PATH': app_dic['icon_path'],
            'CERTIFICATE_ANALYSIS': cert_dic,
            'PERMISSIONS': man_an_dic['permissions'],
            'MALWARE_PERMISSIONS': man_an_dic['malware_permissions'],
            'MANIFEST_ANALYSIS': man_an_dic['manifest_anal'],
            'BINARY_ANALYSIS': bin_anal,
            'FILE_ANALYSIS': app_dic['file_analysis'],
            'ANDROID_API': code_an_dic['api'],
            'CODE_ANALYSIS': code_an_dic['findings'],
            'NIAP_ANALYSIS': code_an_dic['niap'],
            'PERMISSION_MAPPING': code_an_dic['perm_mappings'],
            'URLS': code_an_dic['urls'],
            'DOMAINS': code_an_dic['domains'],
            'EMAILS': code_an_dic['emails'],
            'STRINGS': code_an_dic['strings'],
            'FIREBASE_URLS': code_an_dic['firebase'],
            'FILES': app_dic['files'],
            'EXPORTED_COUNT': man_an_dic['exported_cnt'],
            'APKID': apk_id,
            'QUARK': code_an_dic['behaviour'],
            'TRACKERS': trackers,
            'PLAYSTORE_DETAILS': app_dic['playstore'],
            'NETWORK_SECURITY': man_an_dic['network_security'],
            'SECRETS': code_an_dic['secrets'],
            'SBOM': code_an_dic['sbom'],
        }
        if update_type == 'save':
            db_entry = StaticAnalyzerAndroid.objects.filter(
                MD5=app_dic['md5'])
            if not db_entry.exists():
                StaticAnalyzerAndroid.objects.create(**values)
        else:
            StaticAnalyzerAndroid.objects.filter(
                MD5=app_dic['md5']).update(**values)
    except Exception as exp:
        msg = 'Failed to Save/Update Database'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))
    try:
        values = {
            'APP_NAME': app_dic['real_name'],
            'PACKAGE_NAME': man_data_dic['packagename'],
            'VERSION_NAME': man_data_dic['androvername'],
        }
        RecentScansDB.objects.filter(
            MD5=app_dic['md5']).update(**values)
    except Exception as exp:
        msg = 'Updating RecentScansDB table failed'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))


def save_get_ctx(app, man, m_anal, code, cert, elf, apkid, trk, rscn):
    # SAVE TO DB
    if rscn:
        msg = 'Updating Database...'
        logger.info(msg)
        append_scan_status(app['md5'], msg)
        action = 'update'
        update_scan_timestamp(app['md5'])
    else:
        msg = 'Saving to Database'
        logger.info(msg)
        append_scan_status(app['md5'], msg)
        action = 'save'
    save_or_update(
        action,
        app,
        man,
        m_anal,
        code,
        cert,
        elf,
        apkid,
        trk,
    )
    return get_context_from_analysis(
        app,
        man,
        m_anal,
        code,
        cert,
        elf,
        apkid,
        trk,
    )



import logging
import os
from pathlib import Path
import subprocess
import time

import requests

from django.conf import settings

from mobsf.MobSF.utils import upstream_proxy

logger = logging.getLogger(__name__)


def stop_httptools(url):
    """Kill httptools."""
    # Invoke HTTPtools UI Kill Request
    try:
        requests.get(f'{url}/kill', timeout=5)
        logger.info('Killing httptools UI')
    except Exception:
        pass

    # Invoke HTTPtools Proxy Kill Request
    try:
        http_proxy = url.replace('https://', 'http://')
        headers = {'httptools': 'kill'}
        url = 'http://127.0.0.1'
        requests.get(
            url,
            timeout=5,
            headers=headers,
            proxies={'http': http_proxy})
        logger.info('Killing httptools Proxy')
    except Exception:
        pass


def start_proxy(port, project):
    """Start HTTPtools in Proxy Mode."""
    argz = ['httptools',
            '-m', 'capture',
            '-p', str(port), '-n', project]
    proxies, _ = upstream_proxy('http')
    if proxies['http']:
        argz.extend(['-u', proxies['http']])
    fnull = open(os.devnull, 'w')
    subprocess.Popen(argz, stdout=fnull, stderr=subprocess.STDOUT)


def start_httptools_ui(port):
    """Start Server UI."""
    subprocess.Popen(['httptools',
                      '-m', 'server', '-p', str(port)])
    time.sleep(3)


def create_ca():
    """Generate CA on first run."""
    argz = ['mitmdump', '-n']
    subprocess.Popen(argz,
                     stdin=None,
                     stdout=None,
                     stderr=None,
                     close_fds=True)
    time.sleep(3)


def get_ca_file():
    """Get CA Dir."""
    from mitmproxy import options
    ca_dir = Path(options.CONF_DIR).expanduser()
    ca_file = ca_dir / 'mitmproxy-ca-cert.pem'
    if not ca_file.exists():
        create_ca()
    return ca_file.as_posix()


def get_traffic(package):
    web = Path.home() / '.httptools' / 'flows' / f'{package}.flow.txt'
    if web.is_file():
        return web.read_text('utf-8', 'ignore')
    return ''


def get_http_tools_url(req):
    """Get httptools URL from request."""
    scheme = req.scheme
    ip = req.get_host().split(':')[0]
    port = settings.PROXY_PORT
    return f'{scheme}://{ip}:{str(port)}'
